<?php

class Auth_AclController extends Base_Controller_Action_Cms
{
    /**
     * @var AclResource
     */
    private $_aclResourceModel = null;

    /**
     * @var Auth_Form_AclRule
     */
    private $_formRules;

    /**
     * @var array
     */
    private $_resourceList;

    /**
     * @var array
     */
    private $_roleList;

    /**
     * @var array
     */
    private $_aclRolesWithoutResourceList;

    /**
     * @var array
     */
    private $_rulesList;

    /**
     * @var
     */
    private $_rulesRoleList;


    public function indexAction()
    {
        $this->_roleList = AclRole::getQuery()->execute();
        $this->_aclRolesWithoutResourceList = AclRole::getGroupRole(AclRole::getQuery()->execute());
        $this->_rulesList = AclRule::getAll();
        $this->_resourceList = Doctrine_Query::create()
            ->from('AclResource o INDEXBY o.id_acl_resource')
            ->leftJoin('o.Resources o2 INDEXBY o2.id_acl_resource')
            ->leftJoin('o2.Resources o3 INDEXBY o3.id_acl_resource')
            ->where('o.id_parent IS NULL')
            ->execute(array(), Doctrine::HYDRATE_ARRAY);

        $this->_rulesRoleList = array();
        foreach($this->_roleList as $role){
            $this->_rulesRoleList[$role['name']] = array();
        }

        foreach($this->_rulesList as $rule){
            $this->_rulesRoleList[$rule['role']][$rule['resource']] = (int)$rule['is_allow'];
        }

        $this->_formRules = new Auth_Form_AclRule(array(
            'roleList' => $this->_roleList,
            'resourceList' => $this->_resourceList,
            'rulesRoleList' => $this->_rulesRoleList,
        ));

        if($this->_request->isPost() && $this->_formRules->isValid($this->_request->getPost())){
            $this->_saveRules();
            Base_Cache::clean();

            if($this->_request->isXmlHttpRequest()){
                $roleRules = $this->_getRoleRulesList();
                $returnData = array(
                    'gritter_title' => $this->view->translate('gritter-title_record_save'),
                    'gritter_text' => $this->view->translate('gritter-text_record_save'),
                    'callback_function' => 'loadRoleAndRulePreview('.json_encode($roleRules).')',
                );

                $this->_helper->json($returnData);
            }else{
                $this->_flash()->success->addMessage('label_cms_save_success');
                $this->_redirector()->gotoRouteAndExit();
            }
        }

        $countResourceItemWithoutResource = AclResourceItem::getQuery()->addWhere('o.id_acl_resource IS NULL')->count();

        $this->view->form = $this->_formRules;
        $this->view->aclRolesList = $this->_roleList;
        $this->view->aclRolesWithoutResourceList = $this->_aclRolesWithoutResourceList;
        $this->view->countResourceItemWithoutResource = $countResourceItemWithoutResource;
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_auth_rule') );
    }


    public function newResourceAction()
    {
        $this->_aclResourceModel = new AclResource();

        $form = new Auth_Form_AclResource(array('model' => $this->_aclResourceModel));
        if($this->_request->isPost() && $form->isValid($this->_request->getPost())){
            $this->_aclResourceModel->save();
            $this->_flash()->success->addMessage('label_cms_save_success');
            $this->_redirector()->gotoRouteAndExit(array(), 'auth_acl');
        }

        $this->view->placeholder('page-title')->set($this->view->translate('h1-cms_user-acl-resource_new-form'));
        $this->_helper->viewRenderer('admin/form', null, array('noController' => true));
        $this->view->form = $form;
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_auth_new-resource') );
    }


    public function deleteResourceAction()
    {
        $this->_aclResourceModel = AclResource::findRecordAll($this->getParam('id_acl_resource'));
        $this->forward404Unless($this->_aclResourceModel);

        $this->_aclResourceModel->delete();

        $this->_flash()->success->addMessage('label_cms_delete_success');
        $this->_redirector()->gotoRouteAndExit(array(), 'auth_acl');
    }


    public function loadResourceAction()
    {
        $acl = Base_Acl::getInstance();
        $resourceItemList = $acl->getResourceItems();
        $resourceItemModules = $this->_getResourcesFromModules();
        $count = 0;

        foreach($resourceItemModules as $item){
            if(!isset($resourceItemList[$item])){
                $resourceItem = AclResourceItem::findOneByName($item);
                if(!$resourceItem){
                    $resourceItem = new AclResourceItem();
                    $resourceItem->name = $item;
                    $resourceItem->save();

                    $count++;
                }
            }
        }

        if($count>0){
            $this->_flash()->success->addMessage('flash-cms_acl-resource_load');
        }else{
            $this->_flash()->success->addMessage('flash-cms_acl-resource_empty');
        }

        $this->_redirector()->gotoRouteAndExit(array(), 'auth_acl');

    }


    private function _getResourcesFromModules()
    {
        $frontController = Zend_Controller_Front::getInstance();
        $modules = $frontController->getControllerDirectory();
        $result = array();

        foreach($modules as $module => $path){
            $result[] = $module;

            if (is_dir($path)) foreach (scandir($path) as $file) {
                if ($file{0} !== '.' && strpos($file, '.php') === strlen($file) - 4) {
                    $controller = Base::camelToDash(substr($file, 0, -14));
                    $result[] = $module.'_'.$controller;

                    require_once $path . '/' . $file;

                    $class_name = ($module === 'default' ? '' : Base::dashToCamel($module) . '_') . substr($file, 0, -4);
                    foreach (get_class_methods($class_name) as $method) {
                        if (substr($method,-6) === 'Action') {
                            $action = Base::camelToDash(substr($method, 0, -6));
                            $result[] = $module.'_'.$controller.'_'.$action;
                        }
                    }
                }
            }
        }

        return $result;
    }


    private function _getRoleRulesList()
    {
        $result = array();
        $_resourceList = AclResource::getListName();
        $acl = Base_Acl::_();
        $acl->loadRolesResourcesRules();

        foreach($this->_roleList as $role){
            foreach($_resourceList as $id_resource => $resource_name){

                $result[$role['name']][$id_resource] = $acl->hasAccess($resource_name, $role['name']);
            }
        }

        return $result;
    }


    private function _saveRules()
    {
        $rulesRequest = $this->_getParam($this->_formRules->getElementsBelongTo());
        $rulesRequest = $rulesRequest['rules'];

        $rulesSave = array();
        foreach($this->_rulesRoleList as $role => $rules){
            $rulesSave[$role] = array_intersect_key($rulesRequest[$role], $this->_rulesRoleList[$role]);

            $newRules = array_filter($rulesRequest[$role], 'strlen');
            $rulesSave[$role]+= array_diff_key($newRules, $this->_rulesRoleList[$role]);
        }

        try{
            $conn = Doctrine_Manager::connection();
            $conn->beginTransaction();
            foreach($rulesSave as $role => $rules){
                foreach($rules as $resource_name => $allow){
                    $aclRule = AclRule::findOneByRoleAndResource($role, $resource_name);

                    if($allow === ''){
                        if($aclRule) {
                            $aclRule->delete();
                        }
                    }else{
                        if(!$aclRule){
                            $aclRule = new AclRule();
                            $aclRule->role = $role;
                            $aclRule->resource = $resource_name;
                        }
                        $aclRule->is_allow = $allow;
                        $aclRule->save();
                    }
                }
            }

            $conn->commit();
        }catch (Exception $e){
            $conn->rollback();
            throw new Zend_Exception($e);
        }
    }
}
