<?php

class Filemanager_IndexController extends Base_Controller_Action_Cms
{

    const VALIDATE_FILE_SHORT = '/[^a-zA-Z0-9\.\-\_]/';
    const VALIDATE_FILE = '/[^a-zA-Z0-9ęóąśłżźćńĘÓĄŚŁŻŹĆŃ\(\) \.\-\_]/';

    /*
     * Ścieżka od katalogu strony
     */

    private $path;

    /*
     * Ścieżka absolutna strony na serwerze
     */
    private $pathFiles;

    /*
     * Ścieżka absolutna strony na serwerze + katalogu
     */
    private $pathAbsolute;

    /*
     * scieżka do pliku dla przeglądarki
     */
    private $pathBrowser;

    /*
     * zawartość aktualnie wybranego katalogu z $this->path
     */
    private $catalogContent;

    /**
     * @var Zend_Session_Namespace
     */
    private $_session;

    private $selectedFiles = array('operation' => '', 'files_operation' => '', 'files_path' => '', 'files' => array(), 'showOverwriteForm' => false);
    private $imgExt = array('jpg', 'jpeg', 'png', 'gif', 'bmp');
    private $form = array();
    private $mode = 'list';

    public function preDispatch()
    {
        parent::preDispatch();
        $this->setPaths();
        $this->_session = new Zend_Session_Namespace('filemanager');

        $this->getSelectedFiles();
        $this->catalogContent = $this->getCatalogContent();

        $this->mode = $this->_getParam('mode', 'list');
        if($this->mode == 'get' || $this->mode == 'getImage' && $this->_helper->layout->isEnabled()) {
            Base_Layout::setLayout('Admin', 'layout_empty');
            $this->mode = 'get';
        }

        $this->view->path = $this->path;
        $this->view->mode = $this->mode;
    }

    public function postDispatch()
    {
        parent::postDispatch();
    }

    public function indexAction()
    {
        $dirUp = $this->dirUp($this->path);
        $breadcrumbPath = trim($this->path, '/');
        $breadcrumbPath = empty($breadcrumbPath) ? array() : explode('/', $breadcrumbPath);

        // ustawienie widoku listy
        if($this->hasParam('view')) {
            $viewMode = $this->_getParam('view', 'list');
        } elseif($this->_session->__isset('public_list_view')) {
            $viewMode = $this->_session->__get('public_list_view');
        }

        if(empty($viewMode) || !in_array($viewMode, array('list', 'tiles'))) {
            $viewMode = 'list';
        }
        $this->_session->__set('public_list_view', $viewMode);


//        $this->view->headLink()->appendStylesheet('/css/admin/filemenager.css');
        $this->view->headScript()->appendFile('/assets/js/filemanager.js');
        $this->view->headScript()->appendFile('/assets/lib/bootstrap/js/bootstrap-contextmenu.js');
//        $this->view->headScript()->appendFile('/js/jquery/jquery.rightClick.js');
//		$this->view->headLink()->appendStylesheet('/_css/jquery/jquery.Jcrop.css');
//		$this->view->headScript()->appendFile('/_js/jquery/jquery.Jcrop.js' );


        $this->view->catalogContent = $this->catalogContent;
        $this->view->dirUp = $dirUp;
        $this->view->browserPath = $this->pathBrowser;
        $this->view->breadcrumbPath = $breadcrumbPath;
        $this->view->viewMode = $viewMode;
        $this->view->selectedFiles = $this->selectedFiles;
    }


    public function newCatalogAction()
    {
        $this->form = new Filemanager_Form_Catalog(array('destination' => $this->pathAbsolute));

        if($this->_request->isPost() && $this->form->isValid($this->_request->getPost())){
            $this->_flash()->success->addMessage('label_cms_save_success');
            $this->_redirector()->gotoUrlAndExit(Base::url('filemanager', array('mode' => $this->mode)).'?path='.$this->path);
        }
        $this->view->form = $this->form;
    }


    public function newFileAction()
    {
        $this->form = new Filemanager_Form_File(array('destination' => $this->pathAbsolute));
        if($this->_request->isPost() && $this->form->isValid($this->_request->getPost())){
            if ($this->form->file->receive()) {
                $this->_flash()->success->addMessage('label_cms_save_success');
            }else{
                $this->_flash()->error->addMessage('label_cms_save_error');
            }
            $this->_redirector()->gotoUrlAndExit(Base::url('filemanager', array('mode' => $this->mode)).'?path='.$this->path);
        }
        $this->view->form = $this->form;
    }


    public function editNameAction()
    {
        $element = array( 'ext' => '', 'is_file' => false);
        $element['link'] = trim($this->_getParam('element'), '/');
        $element['name'] = trim(substr($element['link'], strrpos($element['link'], '/')), '/');
        $element['path'] = str_replace($element['name'], '', $element['link']);
        $element['full_path'] = $this->pathFiles.DS.$element['path'];
        $filename = $this->pathFiles . DS . $element['link'];


        if(!file_exists($filename) || strlen($element['link']) == 0) {
            $this->_flash()->danger->addMessage('label_cms_filemahager_editname_not-exists');
            $url = Base::url('filemanager').'?path='.$element['path'];
            $this->_redirector()->gotoUrlAndExit($url);
        }


        if(is_file($filename)) {
            $element['ext'] = substr($element['name'], strrpos($element['name'], '.'));
            $element['name'] = substr($element['name'], 0, strrpos($element['name'], '.'));
            $element['is_file'] = true;
        }


        $this->form = new Filemanager_Form_NameEdit(array('element' => $element));

        if($this->_request->isPost() && $this->form->isValid($this->_request->getPost())){
            $this->_flash()->success->addMessage('label_cms_save_success');
            $this->_redirector()->gotoUrlAndExit(Base::url('filemanager').'?path='.$element['path']);

        }


        $this->view->form = $this->form;
    }


    public function fileInfoAction()
    {
        $filename = $this->_getParam('file');
        $filePath = $this->pathAbsolute . DS . $filename;
        $fileInfo = array();

        $fileInfo['exists'] = file_exists($filePath) ? true : false;
        if($fileInfo['exists']) {
            $fileInfo['file'] = $filename;
            $fileInfo['path'] = 'http://' . DOMAIN . $this->pathBrowser . '/' . $filename;
            $fileInfo['type'] = is_dir($filePath) ? 'dir' : 'file';
            $fileInfo['modificated'] = filemtime($filePath);

            if($fileInfo['type'] == 'dir') {
                $fileInfo['isImage'] = false;
//				$fileInfo['countContent'] = $this->countElementsInsideDir($path.$fileName);
            } else {
                $fileInfo['size'] = filesize($filePath);
                $fileInfo['ext'] = strtolower(Base::getFileExt($filePath));
                $fileInfo['isImage'] = in_array(strtolower($fileInfo['ext']), $this->imgExt);
                if($fileInfo['isImage']) {
                    $imgsize = getimagesize($filePath);
                    $fileInfo['width'] = $imgsize[0];
                    $fileInfo['height'] = $imgsize[1];
                    $fileInfo['preview'] = $fileInfo['path'];
                }
            }
        }

        $this->view->fileInfo = $fileInfo;
    }

//=====================================
//		SELECTED FILES OPERATIONS
//=====================================

    private function setPaths()
    {
        $this->path = str_replace('../','',$this->_getParam('path', ''));
        $this->path = trim($this->path, '/');
        $this->pathFiles = APPLICATION_WEB.DS.'files';// $this->getSelectedBaseUser()->getPublicRelativeRepositoryPath(true);
        $this->pathAbsolute = rtrim($this->pathFiles . DS . $this->path, '/');
        if(!is_dir($this->pathAbsolute)){
            $this->path = '';
            $this->pathAbsolute = $this->pathFiles;
        }
        $this->pathBrowser = rtrim('/files/' . $this->path, '/');
    }


    private function setSelectedFiles()
    {
        $avalibleOptions = array(
            '' => '',
            'copy' => 'copy',
            'cut' => 'cut',
            'paste' => 'paste',
            'cancel' => 'cancel',
            'delete' => 'delete',
        );

        $selectedOption = $this->_getParam('selectedOption');
        if(array_key_exists($selectedOption, $avalibleOptions)) {
            $this->selectedFiles['operation'] = $avalibleOptions[$selectedOption];
            if($this->selectedFiles['operation'] != 'paste') {
                $this->selectedFiles['files_operation'] = $avalibleOptions[$selectedOption];
            }
        }

        if($this->hasParam('file')) {
            $filesList = $this->_getParam('file', array());


            if($this->selectedFiles['operation'] != 'paste') {
                $this->selectedFiles['files'] = array();
            }
            foreach($filesList as $file => $value)
            {
                if($value) {
                    $this->selectedFiles['files'][$file] = false;
                }
            }
            if(count($this->selectedFiles['files']) > 0 && $this->selectedFiles['operation'] != 'paste') {
                $this->selectedFiles['files_path'] = $this->path;
            }
        }

        $this->_session->__set('selectedFiles', $this->selectedFiles);
    }

    private function getSelectedFiles()
    {
        if($this->_session->__isset('selectedFiles')) {
            $this->selectedFiles = $this->_session->__get('selectedFiles');
        }

        return $this->selectedFiles;
    }

    private function setEmptySelectedFiles()
    {
        $this->_session->__unset('selectedFiles');
    }


    /**
     * ackja obslugujaca formularze post z operacjami na zaznaczonych plikach
     */
    public function operationAction()
    {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        $this->setSelectedFiles();

        if($this->selectedFiles['operation'] == 'delete') {
            $this->deleteFiles();
        } elseif($this->selectedFiles['operation'] == 'paste') {
            if($this->selectedFiles['files_operation'] == 'copy') {
                $this->pasteFiles();
            } elseif($this->selectedFiles['files_operation'] == 'cut') {
                $this->renameFiles();
            }
        } elseif($this->selectedFiles['operation'] == 'cancel') {
            $this->setEmptySelectedFiles();
        }

        $gotoUrl = $this->view->url(array('mode' => $this->mode), 'filemanager') . '?path=' . $this->path;
        $this->_redirector()->gotoUrlAndExit($gotoUrl,array('prependBase'=>false));
    }

    private function pasteFiles()
    {
        foreach($this->selectedFiles['files'] as $file => $overwrite){
            if(!array_key_exists($file, $this->catalogContent) || $overwrite == true) {
                $elementFrom = $this->pathFiles . DS . $this->selectedFiles['files_path'] . DS . $file;
                $elementTo = $this->pathAbsolute . DS . $file;
                if(is_file($elementFrom)) {
                    Base::copy($elementFrom, $elementTo);
                } elseif(is_dir($elementFrom)) {
                    if(strstr($elementTo, $elementFrom) == false) {
                        Base::copy($elementFrom, $elementTo);
                    } else {
                        $this->_flash()->error->addMessage('message_filemanager_operation-past_error_own-source');
                    }
                }
            } else {
                $this->sessionFiles['showOverwriteForm'] = true;
            }
        }


        $this->_flash()->success->addMessage('message_filemanager_operation-past_success');
        $this->setEmptySelectedFiles();
    }


    private function renameFiles()
    {
        if($this->selectedFiles['files_path'] != $this->path) {
            foreach($this->selectedFiles['files'] as $file => $overwrite){
                if(!array_key_exists($file, $this->catalogContent) || $overwrite == true) {
                    $elementFrom = $this->pathFiles . DS . $this->selectedFiles['files_path'] . DS . $file;
                    $elementTo = $this->pathAbsolute . DS . $file;

                    if(is_dir($elementFrom)) {
                        if(strstr($elementTo, $elementFrom) == false) {
                            rename($elementFrom, $elementTo);
                        } else {
                            $this->_flash()->success->addMessage('message_filemanager_operation-move_error_own-source');
                        }
                    } else {
                        rename($elementFrom, $elementTo);
                    }
                } else {
                    $this->sessionFiles['showOverwriteForm'] = true;
                }
            }
        } else {
            $this->_flash()->success->addMessage('message_filemanager_operation-move_error_the-same');
        }

        $this->_flash()->success->addMessage('message_filemanager_operation-move_success');
        $this->setEmptySelectedFiles();
    }


    private function deleteFiles()
    {
        foreach($this->selectedFiles['files'] as $file => $overwrite){
            $pathAbsolute = $this->pathAbsolute . DS . $file;
            if(file_exists($pathAbsolute)){
                if(is_dir($pathAbsolute)) {
                    Base::removeDirectories($pathAbsolute);
                } elseif(is_file($pathAbsolute)) {
                    unlink($pathAbsolute);
                }
            }
        }

        $this->_flash()->success->addMessage('message_filemanager_operation-delete_success');
        $this->setEmptySelectedFiles();
    }

//=====================================
//		OTHER NEEDED FUNCTION
//=====================================


    protected function getCatalogContent()
    {
        $catalogContent = array();
        if(is_dir($this->pathAbsolute)) {
            $dh = opendir($this->pathAbsolute);
            if($dh) {
                while (($file = readdir($dh)) !== false){
                    if($file != '.' && $file != '..' && substr($file, 0, 1) != '.') {
                        $pos = array('file' => $file, 'type' => '', 'is_image' => false,
                            'ext' => '', 'name' => '', 'size' => '', 'modificated' => '');

                        if(is_dir($this->pathAbsolute . DS . $file)) {
                            $pos['type'] = 'dir';
                            $pos['name'] = $pos['file'];
                            $pos['ext'] = 'folder';
                        } else {
                            $pos['type'] = 'file';
                            $pos['ext'] = Base::getFileExt($file);
                            $pos['name'] = str_replace('.' . $pos['ext'], '', $pos['file']);
                            $pos['size'] = filesize($this->pathAbsolute . DS . $file);
                            $pos['is_image'] = in_array($pos['ext'], $this->imgExt);

                            if($pos['size'] < 1024) {
                                $k = 'B';
                            } elseif($pos['size'] < (1024 * 1024)) {
                                $k = 'KB';
                                $pos['size']/=1024;
                            } elseif($pos['size'] < (1024 * 1024 * 1024)) {
                                $k = 'MB';
                                $pos['size']/=1024 * 1024;
                            } elseif($pos['size'] < (1024 * 1024 * 1024 * 1024)) {
                                $k = 'GB';
                                $pos['size']/=1024 * 1024 * 1024;
                            }
                            $pos['size'] = round($pos['size'], 2) . ' ' . $k;
                        }
                        $pos['modificated'] = filemtime($this->pathAbsolute . DS . $file);

                        $this->mode = $this->_getParam('mode', 'list');

                        if(!($this->mode == 'getImage') || $pos['type'] == 'dir' || ($this->mode == 'getImage' && $pos['type'] == 'file' && in_array($pos['ext'], $this->imgExt))) {
                            $catalogContent[$pos['name'] . ( $pos['type'] == 'file' ? '.' . $pos['ext'] : '' )] = $pos;
                        }
                    }
                }
                closedir($dh);
            }
        }

        $catalogContent = $this->array_msort($catalogContent, array('type' => SORT_ASC, 'file' => SORT_ASC));
        return $catalogContent;
    }


    private function array_msort($array, $cols)  //multisort po parametrze tablicy
    {
        $colarr = array();
        foreach($cols as $col => $order){
            $colarr[$col] = array();
            foreach($array as $k => $row){
                $colarr[$col]['_' . $k] = strtolower($row[$col]);
            }
        }
        $params = array();
        foreach($cols as $col => $order){
            $params[] = &$colarr[$col];
            $order = (array) $order;
            foreach($order as $order_element){
                //pass by reference, as required by php 5.3
                $params[] = &$order_element;
            }
        }

        call_user_func_array('array_multisort', $params);
        $ret = array();
        $keys = array();
        $first = true;
        foreach($colarr as $col => $arr){
            foreach($arr as $k => $v){
                if($first) {
                    $keys[$k] = substr($k, 1);
                }
                $k = $keys[$k];
                if(!isset($ret[$k])) {
                    $ret[$k] = $array[$k];
                }
                $ret[$k][$col] = $array[$k][$col];
            }
            $first = false;
        }
        return $ret;
    }

    protected function dirUp($path)
    {
        $dirup = $path;
        if(substr($dirup, -1, 1) == '/') {
            $dirup = substr($dirup, 0, strlen($dirup) - 1);
        }
        $n = strRpos($dirup, '/');
        $dirup = substr($dirup, 0, $n);
        return $dirup;
    }

}