<?php

/**
 * Abstract class for extension
 */
require_once 'Zend/View/Helper/FormElement.php';


/**
 * Helper to generate a params fields to navigation tree
 *
 * @category   Zend
 * @package    Zend_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class Navigation_View_Helper_FormTreeItemParams extends Zend_View_Helper_FormElement
{
    /**
     * Generates a 'text' element.
     *
     * @access public
     *
     * @param string|array $name If a string, the element name.  If an
     * array, all other parameters are ignored, and the array elements
     * are used in place of added parameters.
     *
     * @param mixed $value The element value.
     *
     * @param array $attribs Attributes for the element tag.
     *
     * @return string The element XHTML.
     */
    public function formTreeItemParams($name, $value = null, $attribs = null)
    {
        $info = $this->_getInfo($name, $value, $attribs);
        extract($info); // name, value, attribs, options, listsep, disable

        // build the element
        $disabled = '';
        if ($disable) {
            // disabled
            $disabled = ' disabled="disabled"';
        }

        $xhtml = '';
        $value = (array) $value;

        $attribs['class'] = trim(@$attribs['class'].' form-control input-sm');


        $xhtml.= '<div id="tree_item_params_values">';
        foreach($value as $k => $v){
            $xhtml.= '<div class="row">';
                $xhtml.= '<div class="col-md-5">';

                $xhtml.= '<input type="text"'
                        . ' name="' . $this->view->escape('name') . '"'
                        . ' id="' . $this->view->escape($id) . '"'
                        . ' value="' . $this->view->escape($k) . '"'
                        . $disabled
                        . $this->_htmlAttribs($attribs)
                        . $this->getClosingBracket();

                $xhtml.= '</div>';
                $xhtml.= '<div class="col-md-5">';

                $xhtml.= '<input type="text"'
                        . ' name="' . $this->view->escape('value') . '"'
                        . ' id="' . $this->view->escape($id) . '"'
                        . ' value="' . $this->view->escape($v) . '"'
                        . $disabled
                        . $this->_htmlAttribs($attribs)
                        . $this->getClosingBracket();

                $xhtml.= '</div>';

                $xhtml.= '<div class="col-md-2">';
                    $xhtml.= '<span class="btn btn-warning btn-xs delete-param-button"><i class="glyphicon glyphicon-remove"></i></span>';
                $xhtml.= '</div>';


            $xhtml.= '</div>';
        }
        $xhtml.= '</div>';

        $xhtml.= '<div id="tree_item_params_add-button">';
            $xhtml.= '<span id="add_tree_item_content" class="btn btn-xs btn-info"><i class="glyphicon glyphicon-plus"></i> '.$this->view->translate('button-cms_navigation_tree-item_add').'</span>';
        $xhtml.= '</div>';

        $xhtml.= '<div id="tree_item_params_content">';
            $xhtml.= '<div class="row margin-buffer">';
                $xhtml.= '<div class="col-md-5">';

                $xhtml.= '<input type="text"'
                    . ' name="' . $this->view->escape('name') . '"'
                    . ' id="' . $this->view->escape($id) . '"'
                    . ' value=""'
                    . $disabled
                    . $this->_htmlAttribs($attribs)
                    . $this->getClosingBracket();

            $xhtml.= '</div>';
            $xhtml.= '<div class="col-md-5">';

                $xhtml.= '<input type="text"'
                    . ' name="' . $this->view->escape('value') . '"'
                    . ' id="' . $this->view->escape($id) . '"'
                    . ' value=""'
                    . $disabled
                    . $this->_htmlAttribs($attribs)
                    . $this->getClosingBracket();

                $xhtml.= '</div>';

            $xhtml.= '<div class="col-md-2">';
                $xhtml.= '<span class="btn btn-warning btn-xs delete-param-button"><i class="glyphicon glyphicon-remove"></i></span>';
            $xhtml.= '</div>';

            $xhtml.= '</div>';
        $xhtml.= '</div>';

        return $xhtml;
    }
}
