<?php
/**
 * Created by JetBrains PhpStorm.
 * User: skotar
 * Date: 21.08.2013
 * Time: 17:50
 * To change this template use File | Settings | File Templates.
 */

class News_IndexController extends Base_Controller_Action_Frontend {

    /**
     * @var News
     */
    private $_news;

    /**
     * @var NewsCategory
     */
    private $_newsCategory = null;

    /**
     * @var $_newsFormComment News_Form_Comment
     */
    private $_newsFormComment;

    /**
     * @var $_newsComments NewsComment
     */
    private $_newsComment;

    public function init()
    {
        parent::init();
        Base_Layout::prependWidget(array('_class' => 'Gallery_Widget_LastPhoto', 'limit' => 6, 'view' => 'last-photo-slider.phtml'), 'left');
        Base_Layout::prependWidget(array('_class' => 'News_Widget_Category'), 'left');
    }

    public function preDispatch(){
        parent::preDispatch();

        if($this->hasParam('id_news_category')){
            $this->_newsCategory = NewsCategory::findRecord($this->getParam('id_news_category'));
        }
    }

    public function indexAction()
    {
        $query = News::getQuery();
        $query->addWhere('(o.publication_start IS NULL OR o.publication_start <= NOW())')
            ->addWhere('(o.publication_end IS NULL OR o.publication_end >= NOW()')
            ->orderBy('o.publication_start DESC, o.created_at DESC');

        if($this->_newsCategory){
            $query->addWhere('o.id_news_category = ?', $this->_newsCategory->getId());
        }

        if(($metaTitle = Setting::getSetting('news.meta_title')) !== ''){ $this->view->headTitle($metaTitle); }
        if(($metaDescription = Setting::getSetting('news.meta_description')) != ''){ $this->view->headMeta()->setName('description',$this->view->escape($metaDescription)); }
        if(($metaKeywords = Setting::getSetting('news.meta_keywords')) != ''){ $this->view->headMeta()->setName('keywords',$this->view->escape($metaKeywords)); }

        $this->view->newsList = $this->_helper->paging($query);
    }

    public function showAction()
    {
        $id_news = $this->getParam('id_news');
        $this->_news = News::findRecord( $id_news );
        $this->forward404Unless( $this->_news );

        $newsCommentList = NewsComment::getQuery(array('id_news' => $id_news))
            ->addWhere('o.id_news_comment_status = ?', array(NewsCommentStatus::STATUS_ACTIVE))
            ->addOrderBy('o.created_at DESC');

        $this->_newsFormComment = new News_Form_Comment();
        if ( $this->_request->isPost() && $this->_newsFormComment->isValid($this->_request->getPost()) ) {
            $data = $this->_newsFormComment->getValues(true);
            $this->_newsComment = new NewsComment();
            $this->_newsComment->News = $this->_news;
            $this->_newsComment->setText($data['text']);
            $this->_newsComment->setIdNewsCommentStatus(NewsCommentStatus::STATUS_NEW);

            if(Base_Auth::getUser()){
                $this->_newsComment->setIdUser(Base_Auth::getUser('id_user'));
            }
            else
            {
                $this->_newsComment->setNick($data['nick']);
                $this->_newsComment->setEmail($data['email']);
            }

            $this->_newsComment->save();

            $this->_flash()->success->addMessage($this->view->translate('label_news_comments_added_success'));
            $this->_redirector()->gotoRouteAndExit();
        }

        if(($metaTitle = $this->_news->getMetaTitle()) !== ''){ $this->view->headTitle($metaTitle); }
        if(($metaDescription = $this->_news->getMetaDescription()) != ''){ $this->view->headMeta()->setName('description',$this->view->escape($metaDescription)); }
        if(($metaKeywords = $this->_news->getMetaKeywords()) != ''){ $this->view->headMeta()->setName('keywords',$this->view->escape($metaKeywords)); }

        $this->view->keywords = explode(",", $metaKeywords);;
        $this->view->newsFormComment = $this->_newsFormComment;
        $this->view->news = $this->_news;
        $this->view->newsCommentList = $this->_helper->paging($newsCommentList, array('limit' => 10));
    }
}