<?php
/**
 * Created by JetBrains PhpStorm.
 * User: skotar
 * Date: 16.08.2013
 * Time: 02:17
 * To change this template use File | Settings | File Templates.
 */

class News_Form_Admin_News extends Base_Form_Horizontal{

    /**
     * @var News
     */
    protected $_model;

    /**
     * @var Doctrine_Collection
     */
    private $_newsCategoryList;

    public function init()
    {
        $fields = array();

        $fields['title'] = $this->createElement('text', 'title', array(
            'label' => $this->_tlabel.'title',
            'allowEmpty' => false,
            'required' => true,
            'filter' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true)
            ),
            'value' => $this->_model->getTitle(),
        ));

        $fields['publication_start'] = $this->createElement('DateTimePicker', 'publication_start', array(
            'label' => $this->_tlabel.'publication_start',
            'allowEmpty' => true,
            'required' => false,
            'value' => $this->_model->getPublicationStart()
        ));

        $fields['publication_end'] = $this->createElement('DateTimePicker', 'publication_end', array(
            'label' => $this->_tlabel.'publication_end',
            'allowEmpty' => true,
            'required' => false,
            'value' => $this->_model->getPublicationEnd()
        ));

        $this->_newsCategoryList = NewsCategory::getListArray();

        $categoryListOptions = array('' => '') + $this->_newsCategoryList;
        $fields['id_news_category'] = $this->createElement('select', 'id_news_category', array(
            'label' => $this->_tlabel.'id_news_category',
            'allowEmpty' => true,
            'required' => false,
            'multiOptions' => $categoryListOptions,
            'validators' => array(
                array('InArray', true, array(array_keys($categoryListOptions)))
            ),
            'select2' => array(
                'allowClear' => true,
                'placeholder' => $this->getView()->translate('label_news_form_news_category_empty'),
            ),
            'placeholder' => $this->getView()->translate('label_news_form_news_category_empty'),
            'value' => $this->_model->id_news_category,
        ));

        $archived_at_bool = $this->_model->getArchivedAt() === NULL ? false : true;

        $fields['archived_at'] = $this->createElement('checkbox', 'archived_at', array(
            'label' => $this->_tlabel.'archived_at',
            'value' => $archived_at_bool,
            'class' => 'iCheck',
        ));

        if($archived_at_bool)
        {
            $fields['archived_at_time'] = $this->createElement('html', 'archived_at_time', array(
                'label' => $this->_tlabel.'archived_at_time',
                'value' => date('Y-m-d H:i', strtotime($this->_model->getArchivedAt())),
                'class' => 'iCheck',
            ));
        }

        $fields['comments_on'] = $this->createElement('checkbox', 'comments_on', array(
            'label' => $this->_tlabel.'comments_on',
            'value' => $this->_model->getCommentsOn(),
            'class' => 'iCheck',
        ));

        $fields['id_news_image'] = $this->createElement('FileImage', 'id_news_image', array(
            'label' => $this->_tlabel.'id_news_image',
            'allowEmpty' => false,
            'class' => 'filestyle',
            'filestyle' => array(
                'buttonText' => $this->getView()->translate('button_browse_file'),
                'size' => 'sm',
                'buttonName' => 'btn-primary',
                'buttonBefore' => true,
            ),
            'validators' => array(
                array('Extension', false, 'jpg, jpeg, png, gif'),
                array('Count', false, 1),
            )
        ));

        if ( !empty($this->_model->id_news_image ) ) {
            $imageDecorator = $fields['id_news_image']->getDecorator('FileImage');
            $imageDecorator->setOptions( array(
                'id_image' => $this->_model->id_news_image,
                'crop' => true,
                'delete' => true,
            ));
        }

        $fields['lead'] = $this->createElement('textarea', 'lead', array(
            'label' => $this->_tlabel.'lead',
            'allowEmpty' => false,
            'required' => true,
            'filter' => array('StringTrim'),
//            'dimension' => 12,
            'rows' => 6,
            'validators' => array(
                array('NotEmpty', true),
                array('StringLength', true, array('max' => 255)),
            ),
            'value' => $this->_model->getLead()
        ));

        $fields['text'] = $this->createElement('wysiwyg', 'text', array(
            'label' => $this->_tlabel.'text',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getText()
        ));

        $this->addDisplayGroup(
            $fields,
            'main',
            array(
                'legend' => $this->_tlabel.'group_std',
            )
        );

//        $this->setAttrib('class', $this->getAttrib('class').' ajax');
        $this->setAttrib('id', $this->getElementsBelongTo());
//        $this->setAttrib('data-replace', '#'.$this->getElementsBelongTo());

        $group_main = $this->getDisplayGroup('main');

        // dodać formularz metatags
        $metaTagForm = new Admin_Form_Metatags(array('model' => $this->_model, 'wrapperClass' => 'wrapper-blue col-md-12'));

        // sets wrapper
        $this->addSubForm($metaTagForm, 'seo');

        $button = $this->createElement('submit', 'submit', array(
            'label' => 'cms_button_save',
            'class' => 'btn-lg',
        ));

        $this->addElement($button);
    }
}