<?php

class Page_Form_Admin_Page extends Base_Form_Horizontal
{

//    protected $_belong_to = 'PageFormAdminPage';
//
//    protected $_tlabel = 'form_page_admin_';

    /**
     * @var Page
     */
    protected $_model;

    public function init()
    {
        $fields = array();
        $fields_map = array();
        $fields_text = array();
        $fields_buttons = array();

        $fields['is_home_page'] = $this->createElement( 'checkbox', 'is_home_page', array(
            'label' => $this->_tlabel.'is_home_page',
            'value' => $this->_model->is_home_page,
            'class' => 'iCheck',
            'size' => 8,
            'label-size' => 4,
        ));

        $fields['title'] = $this->createElement( 'text', 'title', array(
            'label' => $this->_tlabel.'title',
            'required' => true,
            'allowEmpty' => false,
            'filters' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true),
            ),
            'value' => $this->_model->getTitle(),
            'size' => 8,
            'label-size' => 4,
        ));

        $fields['is_active'] = $this->createElement( 'checkbox', 'is_active', array(
            'label' => $this->_tlabel.'is_active',
            'value' => $this->_model->is_active,
            'class' => 'iCheck',
            'size' => 8,
            'label-size' => 4,
        ));

        $fields['uri'] = $this->createElement('text', 'uri', array(
            'label' => $this->_tlabel.'uri',
            'filters' => array('StringTrim'),
            'validators' => array(
                array('StringLength', true, array('max' => 255)),
                array('AlreadyTaken', true, array(
                    'model' => 'PageTranslation',
                    'column' => 'uri',
                    'params' => array(
                        array('id_language', '=', Base_I18n::getLangId()),
                        array('id_page', '!=', $this->_model->getId())
                    ))),
                array('Regex', true, array('pattern' => '/^[(a-zA-Z0-9-_\/\.)]+$/')),
            ),
            'value' => $this->_model->getUri(),
            'size' => 8,
            'label-size' => 4,
        ));

        $fields['layout'] = $this->createElement( 'select', 'layout', array(
            'label' => $this->_tlabel.'layout',
            'required' => true,
            'allowEmpty' => false,
            'filters' => array('StringTrim'),
            'multioptions' => Base_Layout::getAllLayoutCmsNames(),
            'value' => $this->_model->getDataMapModule().'-'.$this->_model->getDataMapLayout(),
            'size' => 8,
            'label-size' => 4,
        ));

        $fields['id_layout_edit'] = $this->createElement( 'checkbox', 'is_layout_edit', array(
            'label' => $this->_tlabel.'is_layout_edit',
            'value' => $this->_model->is_layout_edit,
            'class' => 'iCheck',
            'size' => 8,
            'label-size' => 4,
        ));


        $metaTagForm = new Admin_Form_Metatags(array('model' => $this->_model, 'wrapperClass' => 'wrapper-blue col-md-6 panel-primary'));

        $metaTagForm->getElement('meta_title')->setAttrib('size', '9')->setAttrib('label-size', '3');
        $metaTagForm->getElement('meta_keywords')->setAttrib('size', '9')->setAttrib('label-size', '3');
        $metaTagForm->getElement('meta_description')->setAttrib('size', '9')->setAttrib('label-size', '3');


        $dataMap = $this->_model->getDataMap();
        $fields_map['placeholders'] = new Page_Form_Element_PageDataMap('data_map', array(
            'label' => $this->_tlabel.'placeholders',
            'value' => $dataMap,
            'belongsTo' => $this->_belong_to,
        ));


        $dm_value = $this->_model->getDataMapPlaceholders();
        $fields_text['text'] = $this->createElement('wysiwyg', 'text', array(
            'label' => $this->_tlabel.'text',
            'filters' => array('StringTrim'),
            'size' => 10,
            'label-size' => 2,
            'value' => @$dm_value['main']['widgets'][0]['html']
        ));


        $this->addDisplayGroup( $fields, 'main', array(
            'legend' => $this->_tlabel.'group_page_info',
            'class' => 'col-md-6 no-bottom-space',
        ));

        $this->addSubForm($metaTagForm, 'seo');

        $group_main = $this->getDisplayGroup('main');
        $this->addHtmlTag(array($group_main, $metaTagForm), array('class' => 'row'));

        $this->addDisplayGroup( $fields_map, 'map',  array(
            'legend' => $this->_tlabel.'group_map',
            'class' => 'col-md-12 panel-danger no-bottom-space group-page-map',
        ));

        $this->addDisplayGroup( $fields_text, 'main_text',  array(
            'legend' => $this->_tlabel.'group_text',
            'class' => 'col-md-12 group-page-text',
        ));

        $group_map  = $this->getDisplayGroup('map');
        $group_text = $this->getDisplayGroup('main_text');
        $group_main = $this->getDisplayGroup('main');

        $group_text->removeDecorator('panel');
        $group_text->removeDecorator('body');

        $this->addHtmlTag(array($group_main, $metaTagForm, $group_text), array('class' => 'col-md-12'));
        $this->addHtmlTag(array($group_main, $metaTagForm, $group_text), array('class' => 'row'));
        $this->addHtmlTag(array($group_map), array('class' => 'row '));

        $fields_buttons['submit'] = $this->createElement('submit', 'submit', array(
            'label' => 'cms_button_save',
            'dimension' => 3,
            'class' => 'btn-lg'
        ));

        $this->addElements($fields_buttons);
    }


    public function render(Zend_View_Interface $view = null)
    {
        $is_layout_edit = $this->getValue('is_layout_edit');

        if($is_layout_edit){
            $group_text = $this->main_text->getDecorator('box');
            $group_text->setOption('class', $group_text->getOption('class') . ' group-hidden');
        }else{
            $group_map = $this->map->getDecorator('box');
            $group_map->setOption('class', $group_map->getOption('class') . ' group-hidden');
        }

        return parent::render($view);
    }

    protected function postIsValid($data)
    {
        $placeholders = array();

        $layout = explode('-', $this->getValue('layout'));
        $this->_model->setDataMapModule(array_shift($layout));
        $this->_model->setDataMapLayout(trim(join('-', $layout), '-'));

        $dm_value = $this->getValue('data_map');

        $dm_value['placeholders'] = isset($dm_value['placeholders']) ? $dm_value['placeholders'] : array();
        $is_layout_edit = $this->getValue('is_layout_edit');

        if($is_layout_edit){
            foreach($dm_value['placeholders'] as $k => $v){
                if(isset($v['widgets'])){
                    foreach( $v['widgets'] as $vv){
                        $placeholders[$k]['widgets'][] = $vv;
                    }
                }else{
                    $placeholders[$k] = $v;
                }
            }
        }else{
            $placeholders['main'] = array(
                'widgets' => array( array(
                    '_class' => 'Page_Widget_Wysiwyg',
                    'html' => $this->getValue('text')
                ))
            );
        }

        $this->_model->setDataMapPlaceholders($placeholders);

        return parent::postIsValid($data);
    }
}