<?php

class Page_Form_Contact extends Base_Form_Horizontal
{

    public function init()
    {
        $fieldsLeft = array();
        $fieldsRight = array();

        $this->getView()->headScript()->appendFile('/assets/lib/bootstrap/js/bootstrap-ajax.js');
        $this->setAction(Base::url('page_contact_form'));

        $fieldsLeft['name'] = $this->createElement('text', 'name', array(
            'placeholder' => $this->getView()->translate($this->_tlabel.'name'),
            'allowEmpty' => false,
            'required' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true)
            ),
            'size' => 12,
        ));

        $fieldsLeft['email'] = $this->createElement('text', 'email', array(
            'placeholder' => $this->getView()->translate($this->_tlabel.'email'),
            'allowEmpty' => false,
            'required' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true),
                array('EmailAddress', true)
            ),
            'size' => 12,
        ));

        $fieldsLeft['phone'] = $this->createElement('text', 'phone', array(
            'placeholder' => $this->getView()->translate($this->_tlabel.'phone'),
            'allowEmpty' => true,
            'required' => false,
            'filters' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true),
            ),
            'size' => 12,
        ));

        $fieldsRight['text'] = $this->createElement('textarea', 'text', array(
            'placeholder' => $this->getView()->translate($this->_tlabel.'text'),
            'allowEmpty' => false,
            'size' => 12,
            'rows' => 5,
        ));

//        $fieldsRight['captcha'] = $this->createElement('captcha', 'captcha', array(
//            'label' => '',
//            'captcha' => array(
//                'captcha' => 'Image',
//                'font' => ROOT_PATH . '/public/assets/fonts/open-sans/OpenSans-Bold.ttf',
//                'width' => 225,
//                'height' => 75,
//                'dotNoiseLevel' => 50,
//                'lineNoiseLevel' => 5,
//                'wordLen' => 6,
//                'imgUrl' => 'http://' . DOMAIN . '/cached/captcha/',
//                'imgDir' => ROOT_PATH . '/public/cached/captcha/',
//            ),
//            'class' => 'form-control captcha',
//            'placeholder' => $this->getView()->translate('label_captcha_placeholder_text'),
//        ));
//
//        $fieldsRight['captcha']->removeDecorator('ViewHelper');

        $fieldsRight['submit'] = $this->createElement('submit', 'submit', array(
            'label' => $this->_tlabel.'submit',
            'buttonType' => 'primary'
        ));

        $filedSize = $fieldsRight['submit']->getDecorator('FieldSize');
        $filedSize->setOption('class', 'col-md-12 text-right');

        $this->addDisplayGroup($fieldsLeft + $fieldsRight, 'main', array(
        ));

        $group = $this->getDisplayGroup('main');
        $group->removeDecorator('body');
        $group->removeDecorator('Panel');
        $group->removeDecorator('box');

        $this->addHtmlTag($fieldsLeft, array('class' => 'col-md-5'));
        $this->addHtmlTag($fieldsRight, array('class' => 'col-md-7'));
        $this->addHtmlTag(array_merge($fieldsLeft, $fieldsRight), array('class' => 'row'));

        $this->setAttrib('class', $this->getAttrib('class').' ajax');
        $this->setAttrib('id', $this->getElementsBelongTo());
        $this->setAttrib('data-replace', '#'.$this->getElementsBelongTo());
    }


    protected function postIsValid($data)
    {
        parent:: postIsValid($data);

        $this->getView()->contactData = $this->getValues(true);

        $mail = new Base_Mail();
        $mail->setBodyContent('contact-form/_mail_contact.phtml');
        $mail->setSubject( $this->getView()->translate('mail_contact_subject').' '.DOMAIN);
        $mail->addTo(Setting::getSetting('email.to'));

        $mail->send();

        if(class_exists('Inquiry_Manager'))
        {
            Inquiry_Manager::add($mail->getSubject(),
                array(
                    'name' => $this->name->getValue(),
                    'email' => $this->email->getValue(),
                    'phone' => $this->phone->getValue(),
                    'text' => $this->text->getValue()
                ),
                'contact_form');
        }

        return true;
    }

}