<?php

class TextPage_AdminController extends Base_Controller_Action_Cms
{
    /**
     * @var TextPage
     */
    private $_textPage;

    private $_mode = null;

    protected $_modelMassName = 'TextPage';

    protected $_massActionMode = 'off';

    protected $_massActionModeType = array('on','off');

    protected $_route_return = 'text-page_admin';

    protected function _getReturnParams()
    {
        return array('id_text_page' => $this->_textPage->getIdTextPageParent());
    }

    public function indexAction()
    {
        $this->_mode = $this->getParam('mode');
        if($this->_mode == 'get'){
            $this->_navLeftDisplay = false;
            Base_Layout::setLayout('Admin', 'layout_empty');
        }

        $data = array('id_text_page_parent' => null, 'left_join' => '');
        if ($this->getParam('id_text_page') ) {
            $this->_textPage = TextPage::findRecordAll($this->getParam('id_text_page'));
            if ( $this->_textPage ) {
                $data['id_text_page_parent'] = $this->_textPage->getId();
            }
        }

        $textPageFilter = new TextPage_Form_Filter();
        $data = $data + $textPageFilter->getValues($textPageFilter->getElementsBelongTo());

        if($this->getParam('mass_action_mode'))
        {
            $this->_massActionMode = $this->getParam('mass_action_mode');
        }

        if(!in_array($this->_massActionMode,$this->_massActionModeType))
        {
            $this->_massActionMode = 'off';
        }

        $query = TextPage::getQuery($data);

        if($data['id_text_page_parent'] === NULL){
            $query->addWhere('o.id_text_page_parent IS NULL');
        }

        $query->addSelect('COUNT(p.id_text_page_parent) as count_pages')
            ->leftJoin('o.Pages p')
            ->groupBy('o.id_text_page');

        $this->_modelMassActions = array(
            'active' => $this->view->translate('label_mass-action_active'),
            'disActive' => $this->view->translate('label_mass-action_disactive'),
            'delete' => $this->view->translate('label_mass-action_delete')
        );

        $this->view->textPageList = $this->_helper->paging($query);
        $this->view->formFilter = $textPageFilter;
        $this->view->textPage = $this->_textPage;
        $this->view->mode = $this->_mode;
        $this->view->actionMode = $this->_massActionMode;
        $this->view->modelMassActions = $this->_modelMassActions;
    }

    public function newAction()
    {
        $this->_textPage = new TextPage();
        if($this->getParam('id_text_page')){
            $textPage = TextPage::findRecordAll($this->getParam('id_text_page'));
            if($textPage){ $this->_textPage->Parent = $textPage; }
        }

        $this->_form($this->_textPage, 'TextPage_Form_TextPage');
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_text-page_new') );
        $this->view->text_page = $this->_textPage;
        $this->_helper->viewRenderer('admin/form');
    }

    public function editAction()
    {
        $this->_textPage = TextPage::findRecordAll($this->getParam('id_text_page'));
        $this->forward404Unless($this->_textPage);

        $this->_form($this->_textPage, 'TextPage_Form_TextPage');
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_text-page_edit') );
        $this->view->text_page = $this->_textPage;
        $this->_helper->viewRenderer('admin/form');
    }

    public function deleteAction()
    {
        $this->_textPage = TextPage::findRecordAll($this->getParam('id_text_page'));
        $this->forward404Unless($this->_textPage);

        $this->_textPage->delete();

        $this->_flash()->success->addMessage('label_text-page_admin_delete_msg-ok');
        $this->_redirector()->gotoRouteAndExit(array('id_text_page' => $this->_textPage->getIdTextPageParent()), 'text-page_admin');
    }
}
