<?php

class TextPage_Form_TextPage extends Base_Form_Horizontal
{

    /**
     * @var TextPage
     */
    protected $_model;

    public function init()
    {
        $fields = array();

        $fields['title'] = $this->createElement( 'text', 'title', array(
            'label' => $this->_tlabel.'title',
            'required' => true,
            'allowEmpty' => false,
            'filters' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true),
                array('StringLength', true, array('max' => 255))
            ),
            'value' => $this->_model->getTitle(),
        ));

        $fields['is_active'] = $this->createElement( 'checkbox', 'is_active', array(
            'label' => $this->_tlabel.'is_active',
            'value' => $this->_model->isActive(),
            'class' => 'iCheck',
        ));

        $paramsUri = array(array('id_language', '=', Base_I18n::getLangId()));
        if(!$this->_model->isNew()){
            $paramsUri[] = array('id_text_page', '!=', $this->_model->getId());
        }

        $fields['uri'] = $this->createElement('text', 'uri', array(
            'label' => $this->_tlabel.'uri',
            'filters' => array('StringTrim'),
            'validators' => array(
                array('StringLength', true, array('max' => 255)),
                array('AlreadyTaken', true, array(
                    'model' => 'TextPageTranslation',
                    'column' => 'uri',
                    'params' => $paramsUri,
                )),
                array('Regex', true, array('pattern' => '/^[(a-zA-Z0-9-_\/\.)]+$/')),
            ),
            'value' => $this->_model->getUri()
        ));


        $fields['lead'] = $this->createElement('textarea', 'lead', array(
            'label' => $this->_tlabel.'lead',
            'filters' => array('StringTrim'),
            'rows' => 6,
            'validators' => array(
                array('StringLength', true, array('max' => 255)),
            ),
            'value' => $this->_model->getLead()
        ));

        $fields['text'] = $this->createElement('wysiwyg', 'text', array(
            'label' => $this->_tlabel.'text',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getText()
        ));


        $this->addDisplayGroup($fields, 'main', array(
            'legend' => $this->_tlabel.'group_std',
        ));

        $metaTagForm = new Admin_Form_Metatags(array('model' => $this->_model));
        $this->addSubForm($metaTagForm, 'seo');

        $buttons['submit'] = $this->createElement('submit', 'submit', array(
            'label' => 'cms_button_save',
            'dimension' => 3
        ));

        $this->addElements($buttons);

    }


}