<?php

class User_AdminController extends Base_Controller_Action_Cms
{

    /**
     * @var User
     */
    private $_user;

    /**
     * @var CompanyHasUser
     */
    private $_modelCompanyHasUser;

    /**
     * @var User_Form_Admin_User
     */
    private $_formUser;

    /**
     * @var User_Form_Admin_Emails
     */
    private $_formEmails;

    /**
     * @var User_Form_Admin_AddEmail
     */
    private $_formAddEmail;

    protected $_modelMassName = 'User';

	public function indexAction()
    {
        $userFilter = new User_Form_Admin_Filter();
        $data = $userFilter->getValues($userFilter->getElementsBelongTo());

        $data['mainEmail'] = '';

        $this->_modelMassActions = array(
            'archive' => $this->view->translate('label_mass-action_archive'),
            'unarchive' => $this->view->translate('label_mass-action_unarchive'),
        );

        $query = User::getQuery($data);

        $this->view->userList = $this->_helper->paging($query);
        $this->view->formFilter = $userFilter;
        $this->view->modelMassActions = $this->_modelMassActions;
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_user_list') );
	}


    public function newAction()
    {
        $this->_user = new User();
        $this->_user->role = AclRole::ROLE_USER;

        $this->_formUser();

        $this->_helper->viewRenderer('form');
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_user_new') );
    }


    public function editAction()
    {
        $this->_user = User::findRecordAll($this->getParam('id_user'));
        $this->forward403Unless($this->_user);

        $this->_formUser();
        $this->_formEmails();
        $this->_formAddEmail();
        $this->_helper->viewRenderer('form');
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_user_edit') );
        $this->view->user = $this->_user;
    }


    private function _formUser()
    {
        $this->_formUser = new User_Form_Admin_User(array('model' => $this->_user));
        $this->view->formUser = $this->_formUser;

        if($this->_request->isPost() &&
            $this->hasParam($this->_formUser->getElementsBelongTo()) &&
            $this->_formUser->isValid($this->_request->getPost())
        ){
            $this->_user->save();
            $this->_flash()->success->addMessage('label_cms_save_success');
            $this->_redirector()->gotoRouteAndExit(array(), 'user_admin');
        }
    }

    private function _formEmails()
    {
        $this->_formEmails = new User_Form_Admin_Emails(array('model' => $this->_user));
        $this->view->formEmails = $this->_formEmails;

        if($this->_request->isPost() &&
            $this->hasParam($this->_formEmails->getElementsBelongTo()) &&
            $this->_formEmails->isValid($this->_request->getPost())
        ){
            $this->_flash()->success->addMessage('label_cms_save_success');
            $this->_redirector()->gotoRouteAndExit();
        }
    }

    private function _formAddEmail()
    {
        $this->_formAddEmail = new User_Form_Admin_AddEmail(array('model' => $this->_user));
        $this->view->formAddEmail = $this->_formAddEmail;

        if($this->_request->isPost() &&
            $this->hasParam($this->_formAddEmail->getElementsBelongTo()) &&
            $this->_formAddEmail->isValid($this->_request->getPost())
        ){
            $this->_flash()->success->addMessage('label_cms_save_success');
            $this->_redirector()->gotoRouteAndExit();
        }
    }


    public function deleteAction($records = null)
    {
        $this->_user = User::findRecordAll($this->_getParam('id_user'));
        $this->forward404Unless($this->_user);

        $id_user = Base_Auth::getUser('id_user');
        if ( !empty( $id_user ) && $this->_user['id_user'] != $id_user ) {
            $this->_user->delete();
            $this->_flash()->success->addMessage('label_user_admin_delete_msg-ok');
        } else {
            $this->_flash()->warning->addMessage('label_user_admin_delete_msg-error');
        }

        $this->_redirector()->gotoRouteAndExit(array(), 'user_admin');
    }


    public function deleteEmailAction()
    {
        $userEmail = UserEmail::findOneByEmail($this->getParam('email'));
        $this->forward404Unless($userEmail);
        $this->forward403If($userEmail->is_main);

        $id_user = $userEmail->id_user;
        $userEmail->delete();
        if ($this->getRequest()->isXmlHttpRequest()) {
            $this->_helper->json(array(
                'status' => 1,
                'gritter_title' => $this->view->translate('label_gritter_title_done'),
                'gritter_text' => $this->view->translate('label_user_admin_delete-email_msg-ok'),
            ));
        }else{
            $this->_flash()->success->addMessage('label_user_admin_delete-email_msg-ok');
            $this->_redirector()->gotoRouteAndExit(array('id_user' => $id_user), 'user_admin_edit');
        }

    }

    public function archiveAction()
    {
        $this->_model = User::findRecordAll($this->_getParam('id_user'));
        $this->forward403Unless($this->_model);

        $this->_model->archived_at = date("Y-m-d H:i:s");
        $this->_model->save();

        $this->_flash()->success->addMessage('label_cms_archive_success');
        $this->_redirector()->gotoRouteAndExit(array(), 'user_admin');
    }

    public function unarchiveAction()
    {
        $this->_model = User::findRecordAll($this->_getParam('id_user'));
        $this->forward403Unless($this->_model);

        $this->_model->archived_at = null;
        $this->_model->save();

        $this->_flash()->success->addMessage('label_cms_unarchive_success');
        $this->_redirector()->gotoRouteAndExit(array(), 'user_admin');
    }
}
