<?php

class User_RegisterController extends Base_Controller_Action_Frontend
{

    /**
     * @var User
     */
    private $_user;

    /**
     * @var User_Form_Register
     */
    private $_formRegister;

    /**
     * @var User_Form_ResendActivate
     */
    private $_formResendActivate;

    /**
     * @var UserConfirmRequest
     */
    private $_userConfirmRequest;

    public function indexAction()
    {
        $this->_user = new User();
        $this->_user->role = AclRole::ROLE_USER;

        $this->_formRegister = new User_Form_Register(array('user' => $this->_user));
        if($this->_request->isPost() && $this->_formRegister->isValid($this->_request->getPost())){
            $this->_user->save();
            $this->_sendRegisterConfirmEmail();
            $this->_redirector()->gotoRouteAndExit(array(), 'user_register_thx');
        }

        $this->view->form = $this->_formRegister;
    }


    public function thxAction(){}


    private function _sendRegisterConfirmEmail()
    {
        $userConfirmRequest = new UserConfirmRequest();
        $userConfirmRequest->User = $this->_user;
        $userConfirmRequest->setHash(Base::getHash());
        $userConfirmRequest->setType(UserConfirmRequest::TYPE_REGISTER);
        $userConfirmRequest->setChannel(get_class($this->_user));
        $userConfirmRequest->setObjectId($this->_user->getId());
        $userConfirmRequest->save();

        $this->view->user = $this->_user;
        $this->view->userConfirmRequest = $userConfirmRequest;

        $mail = new Base_Mail();
        $mail->setBodyContent('register/_mail_register.phtml');
        $mail->setSubject( $this->view->translate('mail_user_register_subject') );
        $mail->addTo($this->_user->getMainEmail()->email, $this->_user->getUserName());
        $mail->send();
    }

    private function _sendRemindConfirmEmail($new = null)
    {
        $this->view->user = $this->_user;
        $this->view->userConfirmRequest = $this->_userConfirmRequest;


        $mail = new Base_Mail();
        if (null === $new) {
            $mail->setBodyContent('register/_mail_remind_link.phtml');
            $mail->setSubject( $this->view->translate( 'mail_user_register_remind-link_subject' ) );
        } else {
            $mail->setBodyContent('register/_mail_register.phtml');
            $mail->setSubject( $this->view->translate( 'mail_user_register_new-link_subject' ) );
        }
        $mail->clearRecipients();
        $mail->addTo( $this->_user->getMainEmail()->email, $this->_user->getUserName() );
        $mail->send();
    }


    public function activateAction()
    {
        $this->_userConfirmRequest = UserConfirmRequest::findOneByHash($this->_getParam('hash'));
        $message = null;
        $result = null;

        if ( !$this->_userConfirmRequest ) {
            $message = 'message_user_register_confirm-activate_error';
            $result = 'error';
        } else {
            $isActive = (bool) $this->_userConfirmRequest->is_active;
            if ( !$isActive ) {
                $message = 'message_auth_index_confirm-activate_info-link-expired';
                $result = 'info';
            } else {
                $this->_userConfirmRequest->setUsedAt(date('YmdHis'));
                $this->_userConfirmRequest->save();

                $email = $this->_userConfirmRequest->User->getMainEmail();
                $email->setIsActive(true);
                $email->save();

                $message = 'message_user_register_confirm-activate_success';
                $result = 'success';
            }
        }

        $this->view->message = $message;
        $this->view->result = $result;
    }

    /**
     * Dodać odpowiedni walidator, czy user ma nie aktywne konto
     */
    public function resendActivateAction()
    {
        $this->_formResendActivate = new User_Form_ResendActivate();

        if ( $this->_request->isPost() && $this->_formResendActivate->isValid( $this->_request->getPost() ) ) {
            $email = $this->_formResendActivate->getValue('email');
            $userEmail = UserEmail::findOneByEmail($email);
            $this->_user = $userEmail->User;
            $message = null;
            $result = null;

            $query = UserConfirmRequest::getQuery(
                array(
                    'type' => UserConfirmRequest::TYPE_REGISTER,
                    'id_user' => $this->_user->getId(),
                    'channel' => get_class($this->_user),
                    'object_id' => $this->_user->getId()
                )
            );
            $this->_userConfirmRequest = $query->fetchOne();

            // if user is_active false || email is_active false
            if ( false === $userEmail->is_active ) {

                if ( !$this->_userConfirmRequest ) {
                    $message = 'There is no activate link for this user';
                    $result = 'error';
                } else {
                    $isActive = (bool) $this->_userConfirmRequest->is_active;
                    if ( $isActive ) {
                        $this->_sendRemindConfirmEmail();
                        $message = 'Activation link was sent to your email address at ' . $this->_userConfirmRequest->created_at;
                        $result = 'info';
                    } else {
                        $this->_userConfirmRequest = new UserConfirmRequest();
                        $this->_userConfirmRequest->User = $this->_user;
                        $this->_userConfirmRequest->setHash(Base::getHash());
                        $this->_userConfirmRequest->setType(UserConfirmRequest::TYPE_REGISTER);
                        $this->_userConfirmRequest->setChannel(get_class($this->_user));
                        $this->_userConfirmRequest->setObjectId($this->_user->getId());
                        $this->_userConfirmRequest->save();

                        $this->_sendRemindConfirmEmail(true);

                        $message = 'New activation link was sent to your email address';
                        $result = 'success';
                    }
                }
            } else {
                $message = 'Your account is activated';
                $result = 'info';
            }
            $this->view->message = $message;
            $this->view->result = $result;

        }

        $this->view->form = $this->_formResendActivate;
    }


}
