<?php

class User_Form_Admin_User extends Base_Form_Horizontal
{
    /**
     * @var array
     */
    protected $_aclRoleList;

    /**
     * @var User
     */
    protected $_model;

    public function init()
    {
        $fields = array();
        $fields_contact = array();
        $fields_company = array();
        $buttons = array();

        $fields['name'] = $this->createElement( 'text', 'name', array(
            'label' => $this->_tlabel.'name',
            'required' => true,
            'allowEmpty' => false,
            'filters' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true)
            ),
            'value' => $this->_model->getName(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields['surname'] = $this->createElement( 'text', 'surname', array(
            'label' => $this->_tlabel.'surname',
            'required' => true,
            'allowEmpty' => false,
            'filters' => array('StringTrim'),
            'validators' => array(
                array('NotEmpty', true)
            ),
            'value' => $this->_model->getSurname(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields['id_profile_image'] = $this->createElement('FileImage', 'id_profile_image', array(
            'label' => $this->_tlabel.'id_profile_image',
            'allowEmpty' => false,
            'validators' => array(
                array('Extension', false, 'jpg, png, gif'),
                array('Count', false, 1),
            ),
            'size' => 9, 'label-size' => 3,
            'filestyle' => array(
                'buttonText' => $this->getView()->translate('button_browse_file'),
                'size' => 'sm',
                'buttonName' => 'btn-primary',
                'buttonBefore' => true,
            ),
        ));

        if(!empty($this->_model->id_profile_image)){
            $imageDecorator = $fields['id_profile_image']->getDecorator('FileImage');
            $imageDecorator->setOptions( array(
                'id_image' => $this->_model->id_profile_image,
                'crop' => true,
                'delete' => true,
            ));
        }

        if($this->_model->getIdUser() != Base_Auth::getUser('id_user'))
        {
            $fields['archived_at'] = $this->createElement('checkbox', 'archived_at', array(
                'label' => $this->_tlabel.'archived_at',
                'value' => (bool)$this->_model->getArchivedAt(),
                'size' => 9, 'label-size' => 3,
                'class' => 'iCheck',
            ));

            if($this->_model->getArchivedAt()){
                $fields['archived_at_date'] = $this->createElement('html', 'archived_at_date', array(
                    'label' => $this->_tlabel.'archived_at_date',
                    'html' => $this->_model->getArchivedAt(),
                    'ignore' => true,
                    'size' => 9, 'label-size' => 3,
                ));
            }
        }

        if($this->_model->isNew()){
            $fields['email'] = $this->createElement( 'text', 'email', array(
                'label' => $this->_tlabel.'email',
                'required' => true,
                'allowEmpty' => false,
                'filters' => array('StringTrim'),
                'validators' => array(
                    array('StringLength', true, array('max' => 255)),
                    array('NotEmpty', true),
                    array('EmailAddress', true),
                    array( 'AlreadyTaken', true, array('model' => 'UserEmail', 'column' => 'email')),
                ),
                'size' => 9, 'label-size' => 3,
            ));
        }


        // password
        $vStringLength = new Zend_Validate_StringLength(array('min' => 6, 'max' => 20));
        $vIdentival = new Zend_Validate_Identical('password');

        $passwordOptions = array(
            'label' => $this->_tlabel.'password',
            'allowEmpty' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array( $vStringLength, true ),
                array( 'NotEmpty', true ) ),
            'size' => 9, 'label-size' => 3,
        );

        $passwordRepeatOptions = array(
            'label' => $this->_tlabel.'repeat-password',
            'allowEmpty' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array( $vStringLength, true ),
                array( $vIdentival, true ),
                array( 'NotEmpty', true ) ),
            'size' => 9, 'label-size' => 3,
        );

        if($this->_model->isNew())
        {
            $passwordOptions['allowEmpty'] = $passwordRepeatOptions['allowEmpty'] = 'false';
            $passwordOptions['required'] = $passwordRepeatOptions['required'] = 'true';
            $passwordOptions['validators'][] = $passwordRepeatOptions['validators'][] = array('NotEmpty', true);
        }

        $fields['password'] = $this->createElement( 'password', 'password', $passwordOptions);
        $fields['password_repeat'] = $this->createElement( 'password', 'password_repeat', $passwordRepeatOptions);

        $localeList = array_merge(array('' => ''), Base_I18n::getLocaleList());
        $fields['locale'] = $this->createElement( 'select', 'locale', array(
            'label' => $this->_tlabel.'locale',
            'allowEmpty' => true,
            'multiOptions' => $localeList,
            'value' => $this->_model->getLocale(),
            'validators' => array(
                array('InArray', true, array(array_keys($localeList)))
            ),
            'select2' => array(
                'allowClear' => true,
                'placeholder' => '',
            ),
            'size' => 9, 'label-size' => 3,
        ));

        $avalibleLanguage = Base_I18n::loadAvailableLanguages();
        $languageOptions = array('' => '');
        foreach($avalibleLanguage['languages'] as $code => $lang){
            $languageOptions[$lang['id_language']] = $lang['name'];
        }

        $fields['id_language'] = $this->createElement( 'select', 'id_language', array(
            'label' => $this->_tlabel.'id_language',
            'allowEmpty' => true,
            'multiOptions' => $languageOptions,
            'value' => $this->_model->getIdLanguage(),
            'validators' => array(
                array('InArray', true, array(array_keys($languageOptions)))
            ),
            'size' => 9, 'label-size' => 3,
        ));


        $this->_aclRoleList = AclRole::getList();
        $roleOptions = array();
        foreach($this->_aclRoleList as $role){
            if($role['name'] != Base_Auth::MAIN_ROLE){
                $roleOptions[$role['name']] = $role['name'];
            }
        }

        $fields['role'] = $this->createElement( 'multiCheckbox', 'role', array(
            'label' => $this->_tlabel.'role',
            'allowEmpty' => true,
            'multiOptions' => $roleOptions,
            'value' => $this->_model->getRoles(),
            'size' => 9, 'label-size' => 3,
            'class' => 'iCheck',
            'inline' => true,
        ));


        $fields_contact['phone'] = $this->createElement( 'text', 'phone', array(
            'label' => $this->_tlabel.'phone',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getPhone(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_contact['street'] = $this->createElement( 'text', 'street', array(
            'label' => $this->_tlabel.'street',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getStreet(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_contact['post_code'] = $this->createElement( 'text', 'post_code', array(
            'label' => $this->_tlabel.'post_code',
            'filters' => array('StringTrim'),
            'validators' => array(
                array('Regex', true, array('/^([0-9]{2})-([0-9]{3})$/'))
            ),
            'value' => $this->_model->getPostCode(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_contact['city'] = $this->createElement( 'text', 'city', array(
            'label' => $this->_tlabel.'city',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getCity(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_company['company_name'] = $this->createElement( 'text', 'company_name', array(
            'label' => $this->_tlabel.'company_name',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getCompanyName(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_company['company_nip'] = $this->createElement( 'text', 'company_nip', array(
            'label' => $this->_tlabel.'company_nip',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getCompanyNip(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_company['company_street'] = $this->createElement( 'text', 'company_street', array(
            'label' => $this->_tlabel.'company_street',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getCompanyStreet(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_company['company_post_code'] = $this->createElement( 'text', 'company_post_code', array(
            'label' => $this->_tlabel.'company_post_code',
            'filters' => array('StringTrim'),
            'validators' => array(
                array('Regex', true, array('/^([0-9]{2})-([0-9]{3})$/'))
            ),
            'value' => $this->_model->getCompanyPostCode(),
            'size' => 9, 'label-size' => 3,
        ));

        $fields_company['company_city'] = $this->createElement( 'text', 'company_city', array(
            'label' => $this->_tlabel.'company_city',
            'filters' => array('StringTrim'),
            'value' => $this->_model->getCompanyCity(),
            'size' => 9, 'label-size' => 3,
        ));


        $this->addDisplayGroup($fields, 'main', array(
            'legend' => $this->_tlabel.'group_user_info',
        ));

        $this->addDisplayGroup($fields_contact, 'contact', array(
            'legend' => $this->_tlabel.'group_user_contact',
        ));

        $this->addDisplayGroup($fields_company, 'company', array(
            'legend' => $this->_tlabel.'group_user_company',
        ));

        $buttons['submit'] = $this->createElement('submit', 'submit', array(
            'label' => 'cms_button_save',
            'offset' => 3,
        ));

        $this->addElements($buttons);
    }

    protected function postIsValid($data)
    {
        if($this->_model->isNew()){
            $userEmail = new UserEmail();
            $userEmail->email = $this->getValue('email');
            $userEmail->is_active = true;
            $userEmail->is_main = true;

            $this->_model->Emails->add($userEmail);
        }

        return parent::postIsValid($data);
    }

}