<?php

class User_Form_Register extends Base_Form_Horizontal
{

    protected $_belong_to = 'UserFormRegister';

    protected $_tlabel = 'user_form_register_';

    /**
     * @var User
     */
    protected $_user;

    /**
     * @param $user User
     */
    protected function setUser($user)
    {
        $this->_user = $user;
    }


    public function init()
    {
        $fields = array();

        $fields['name'] = $this->createElement( 'text', 'name', array(
            'label' => $this->_tlabel.'name',
            'required' => true,
            'filters' => array('StringTrim'),
        ));

        $fields['surname'] = $this->createElement( 'text', 'surname', array(
            'label' => $this->_tlabel.'surname',
            'required' => true,
            'filters' => array('StringTrim'),
        ));

        $fields['email'] = $this->createElement( 'text', 'email', array(
            'label' => $this->_tlabel.'email',
            'required' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array( 'EmailAddress', true),
                array( 'AlreadyTaken', true, array('model' => 'UserEmail', 'column' => 'email')),
            ),
        ));

        $fields['email_repeat'] = $this->createElement( 'text', 'email_repeat', array(
            'label' => $this->_tlabel.'email_repeat',
            'required' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array( 'EmailAddress', true),
                array( 'Identical', true, array('email')),
            ),
        ));

        $fields['password'] = $this->createElement( 'password', 'password', array(
            'label' => $this->_tlabel.'password',
            'required' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array( 'StringLength', true, array('min' => 6, 'max' => 20)),
            ),
        ));

        $fields['password_repeat'] = $this->createElement( 'password', 'password_repeat', array(
            'label' => $this->_tlabel.'repeat-password',
            'required' => true,
            'filters' => array('StringTrim'),
            'validators' => array(
                array( 'StringLength', true, array('min' => 6, 'max' => 20)),
                array( 'Identical', true, array('password') ),
            ),
        ));

        $fields['rules'] = $this->createElement( 'checkbox', 'rules', array(
            'label' => $this->_tlabel.'rules',
            'required' => true,
            'uncheckedValue' => null,
        ));


        $fields['submit'] = $this->createElement('submit', 'submit', array(
            'label' => $this->_tlabel.'submit',
            'class' => 'pull-right'
        ));

        $this->addDisplayGroup(
            $fields,
            'main',
            array(
            )
        );

        $g = $this->getDisplayGroup('main');
        $g->removeDecorator('panel');
    }


    public function postIsValid($data)
    {
        $data = $this->getValues($this->_belong_to);

        $this->_user->setName($data['name']);
        $this->_user->setSurname($data['surname']);

        $userEmail = new UserEmail();
        $userEmail->setEmail($data['email']);
        $userEmail->is_main = true;
        $this->_user->Emails->add($userEmail);

        $this->_user->setPassword($data['password']);

        return true;
    }
}