<?php

class User_Form_Element_UserDataMap extends Zend_Form_Element_Textarea
{

    public $helper = 'formUserDataMap';

    private $_module = null;
    private $_layout = null;
    private $_placeholders = array();

    /**
     * @var User_Validate_UserDataMap
     */
    private $_userDataMapValidator;

    public function __construct($spec, $options = null)
    {
        parent::__construct($spec, $options);

        $this->setBelongsTo($options['belongsTo']);
        $this->createDataMap($this->getValue());

        $this->_userDataMapValidator = new User_Validate_UserDataMap();
        $this->_userDataMapValidator->setPlaceholders($this->_placeholders);
        $this->addValidator($this->_userDataMapValidator, false);
    }


    public function isValid($value, $context = null)
    {
        $this->createDataMap($value);
        $this->_userDataMapValidator->setPlaceholders($this->_placeholders);

        return parent::isValid($value, $context);
    }


    private function createDataMap($value)
    {
        $this->_module = $value['module'];
        $this->_layout = $value['layout'];
        $this->_placeholders = isset($value['placeholders']) ? $value['placeholders'] : array();

        foreach($this->_placeholders as $placeholder => &$definitions)
        {
            if(empty($definitions)) { continue; }
            foreach($definitions['widgets'] as $k => &$widget){
                // dodane $this->_belongsTo
                $widget['_BelongsTo'] = $this->_belongsTo.'[data_map][placeholders]['.$placeholder.'][widgets]['.$k.']';
                if(class_exists($widget['_class'])){
                    $w = new $widget['_class']();
                    $widget['_form'] = $w->form($widget);
                }
            }
        }

        $this->setAttrib('module', $this->_module);
        $this->setAttrib('layout', $this->_layout);
        $this->setAttrib('placeholders', $this->_placeholders);
    }

    /**
     * Adds a file decorator if no one found
     */
    public function loadDefaultDecorators()
    {
        $this->clearDecorators();

        $this->setDecorators(array(
            array('FieldSize'),
            array('ViewHelper'),
//            array('Addon'),
            array('ElementErrors'),
            array('Description', array('tag' => 'p', 'class' => 'help-block')),
            array('HtmlTag', array('tag' => 'div', 'class' => 'placeholders-controls', 'id' => 'placeholders-controls')),
            array('Panel')
        ));

        return $this;
    }

}

