<?php
/**
 * Zend Framework
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @category   Zend
 * @package    Zend_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2005-2011 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id: FormFile.php 23775 2011-03-01 17:25:24Z ralph $
 */


/**
 * Abstract class for extension
 */
require_once 'Zend/View/Helper/FormElement.php';


/**
 * Helper to generate a "file" element
 *
 * @category   Zend
 * @package    Zend_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2005-2011 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class User_View_Helper_FormUserDataMap extends Zend_View_Helper_FormElement
{
    /**
     * Generates a 'file' element.
     *
     * @access public
     *
     * @param string|array $name If a string, the element name.  If an
     * array, all other parameters are ignored, and the array elements
     * are extracted in place of added parameters.
     *
     * @param array $attribs Attributes for the element tag.
     *
     * @return string The element XHTML.
     */
    public function formUserDataMap($name, $value, $attribs = null)
    {
        $info = $this->_getInfo($name, $value, $attribs);
        extract($info);
        $xhtml = '';

        /* check if there is any element map, if not create empty array() */
        if (!isset($value['placeholders'])) $value['placeholders'] = array();

        /** @var $view Zend_View */
        $view = clone Base_Layout::getView();
        $view->clearVars();

        $view->setBasePath(APPLICATION_PATH . '/modules/' . Base::camelToDash($attribs['module']) . '/layouts/');
        $view->addHelperPath(ROOT_PATH.'/library/Base/View/Helper', 'Base_View_Helper');

        $template = Setting::getSetting('template.frontend');
        if(!empty($template)){
            $view->addBasePath(APPLICATION_PATH . '/templates/frontend/' . $template . '/modules/' . Base::camelToDash($attribs['module']) . '/layouts/');
        }

        $view->headLink()->appendStylesheet('/assets/lib/fancybox/source/jquery.fancybox.css');
        $view->headScript()->appendFile('/assets/lib/fancybox/source/jquery.fancybox.js');
        $view->headScript()->appendScript("$('.add-widget').fancybox({width:'85%', minWidth:'780'});");

        $view->headScript()->appendFile('/assets/lib/tinymce/jquery.tinymce.min.js');
        $view->headScript()->appendFile('/assets/js/jsTinyMce.js');
        $view->headScript()->appendFile('/assets/js/map.js');

        $definitions = Base_Layout::getLayoutDefinitions($attribs['module'], $attribs['layout']);

        $i = 0;
        foreach($definitions as $k => $v ){
            $view->placeholder($k)->{$k.'_'.($i++)} = $this->view->partial('admin-dashboard/_user_data_map.phtml', array(
                'map' => (array) @$attribs['placeholders'][$k],
                'placeholder' => $k,
                'module' => @$attribs['module'],
                'layout' => @$attribs['layout'],
                'name' => $name,
                'belongTo' => 'User_Form_Admin_Dashboard',
            ));
        }

        $xhtml.= $this->view->formHidden($name . '[module]', @$attribs['module']);
        $xhtml.= $this->view->formHidden($name . '[layout]', @$attribs['layout']);
        $xhtml.= $view->render($attribs['layout'].'.phtml');

        return $xhtml;
    }
}
