<?php
class Base_Acl extends Zend_Acl
{

//	const DEFAULT_POLICY = 'allow';
	const DEFAULT_POLICY = 'deny';

    /**
	 * @var Base_Acl
	 */
	private static $_acl;

    private $_my_role = null;

    private $_resourceItems = array();

	/**
	 * @return Base_Acl
	 */
	public static function getInstance()
	{
        if ( is_null( self::$_acl ) ) {
            $cache = Base_Cache::getCache();
            self::$_acl = $cache->load('Base_Acl');

            if( !self::$_acl ){
                self::$_acl = new self();
				self::$_acl->loadRolesResourcesRules();
				$cache->save(self::$_acl, 'Base_Acl');
            }

        }
        return self::$_acl;
	}


    /**
     * @return Base_Acl
     */
    public static function _()
    {
        return self::getInstance();
    }

	private function __construct()
	{
	}

	public function loadRolesResourcesRules()
	{
        $this->removeAll();
        $this->removeRoleAll();

		// add basics resource to ACL for framework
		$this->addResource('default');
		$this->addResource('default_error', 'default');
		$this->addResource('default_error_error', 'default_error');
		$this->addResource('default_error_error404', 'default_error');
		$this->addResource('default_error_error503', 'default_error');
		$this->addResource('page');
		$this->addResource('page_index', 'page');
		$this->addResource('auth');
		$this->addResource('auth_index', 'auth');


        // load resources from DB to ACL for framework
        $this->_resourceItems = AclResourceItem::getItemsToResource();
        $resourceList = AclResource::getAll();
        foreach($resourceList as $p1){
            if(!$this->has($p1['name'])){ $this->addResource($p1['name']); }

            foreach(@$p1['Resources'] as $p2){
                if(!$this->has($p2['name'])){ $this->addResource($p2['name'], $p1['name']); }

                foreach(@$p2['Resources'] as $p3){
                    if(!$this->has($p3['name'])){ $this->addResource($p3['name'], $p2['name']); }
                }
            }
        }

        // set basics rules to ACL for framework
        if (self::DEFAULT_POLICY == 'deny') {
            $this->deny();
        } else {
            $this->allow();
        }


        $this->allow(null, array(
            'auth_index',
            'page_index',
            'default_error',
        ));


        // set roles from DB to ACL for framework
        $roles = AclRole::getAll();

        foreach($roles as $r1){
            $this->addRole($r1['name']);

            foreach(@$r1['Roles'] as $r2){
                $this->addRole($r2['name'], $r1['name']);

                foreach(@$r2['Roles'] as $r3){
                    $this->addRole($r3['name'], $r2['name']);

                    foreach(@$r3['Roles'] as $r4){
                        $this->addRole($r4['name'], $r3['name']);
                    }
                }
            }
        }

        // set rules from DB to ACL for framework
        $rules = AclRule::getAll();
		foreach($rules as $rule) {
            try {
				if ($rule['is_allow'] == true) {
					$this->allow($rule['role'], $rule['resource']);
				}else{
					$this->deny($rule['role'], $rule['resource']);
				}
			} catch (Exception $e) {
			}
		}

	}

	public function hasAccess($resource, $role = null)
	{
        if(isset($this->_resourceItems[$resource])) {
            $resource = $this->_resourceItems[$resource];
        }

		if ($this->has($resource) === false)
        {
            $this->_logResourceNoSetAccess($resource);
			return self::DEFAULT_POLICY === 'deny' ? false : true;
		}

        if(empty($role)){
            $role = $this->getMyRole();
        }else{
            $role = $this->getRolesByRole($role);
        }


		return $this->isAllowed($role, $resource);
	}


    public function getMyRole()
	{

        if(null === $this->_my_role){
            $user_roles = Base_Auth::getUserRoles();

            $this->_my_role = md5(time() . serialize($_SERVER));
            $this->addRole($this->_my_role, $user_roles);
        }

		return $this->_my_role;
	}


    public function getRolesByRole($role)
    {
        $roles = array(Base_Auth::MAIN_ROLE, $role);
        $role_name = md5(json_encode($roles));

        if(!$this->hasRole($role_name)){
            $this->addRole($role_name, $roles);
        }

        return $role_name;
    }


    /**
     * @return array
     */
    public function getResourceItems()
    {
        return $this->_resourceItems;
    }


    /**
     * Log resource name when access is not defined
     * @param $resource
     */
    private function _logResourceNoSetAccess($resource)
    {
        $file = APPLICATION_DATA . '/logs/log_resource_no_set_access.log';

        $content = '[' . date('Y-m-d H:i:s') . '] ' ;
        $content .= 'Resource: ' . $resource . PHP_EOL;

        // write content to file
        if(file_exists($file)){
            $content.= file_get_contents($file, false, null, null, 1024*1024); // last 1Mb of log;
        }

        $handle = fopen( $file, 'w' );
        fwrite($handle, $content);
        fclose($handle);
    }
}

