<?php
class Base_Controller_Action_Cms extends Base_Controller_Action
{
    protected $_route_return = null;

    protected $_navLeftDisplay = true;

    /**
     * Settings for mass action nad list/table data
     */

    protected $_modelMassName = null;

    protected $_modelMassActions = null;

    protected $_massActionMode = 'off';

    protected $_massActionModeType = array('on','off');


    /**
     * Settings for query and list/table data
     */
    protected $_listData = array(
        'filter' => array(),
        'id_query' => null,
    );

    protected $_queryFilterClassName = null;

    protected $_queryList = array();



    public function init()
    {
        Base_Layout::setTemplateType('backend');
        Base_Layout::setLayout('Admin', 'layout');
        Base_I18n::setContentLanguage($this->getParam('setContentLanguage'));

        if($this->getParam('mass_action_mode'))
        {
            $this->_massActionMode = $this->getParam('mass_action_mode');
        }

        if(!in_array($this->_massActionMode,$this->_massActionModeType))
        {
            $this->_massActionMode = 'off';
        }

        $this->_modelMassActions = array(
            'active' => $this->view->translate('label_mass-action_active'),
            'disActive' => $this->view->translate('label_mass-action_disactive'),
            'delete' => $this->view->translate('label_mass-action_delete')
        );

        $this->view->actionMode = $this->_massActionMode;
        $this->view->modelMassActions = $this->_modelMassActions;
    }


    public function  preDispatch()
    {
        parent::preDispatch();

        if($this->_queryFilterClassName){
            $this->_prepareQueryData();
        }

    }


    public function  postDispatch()
    {
        parent::postDispatch();

        if(!$this->_helper->viewRenderer->getNoRender()){
            if( $this->_helper->layout->isEnabled() ){
                $this->view->renderToPlaceholder('_nav-main.phtml', 'nav-main');

                if($this->_navLeftDisplay){
                    $navLeft = $this->_request->getControllerName().DS.'_nav-left.phtml';
                    $navLeftFile = APPLICATION_PATH.DS.'modules'.DS.$this->_request->getModuleName().DS.'views'.DS.'scripts'.DS.$navLeft;
                    if(file_exists($navLeftFile)){
                        $this->view->placeholder('page-container-class')->append('container-with-sidebar');
                        $this->view->renderToPlaceholder('_hide_menu.phtml', 'nav-left');
                        $this->view->renderToPlaceholder('_clock.phtml', 'nav-left');
                        $this->view->renderToPlaceholder($navLeft, 'nav-left');
                    }
                }
            }
        }

        $this->_massActionForm();

        if($this->_request->isXmlHttpRequest()){
            Base_Layout::setLayout('Admin', 'layout_overbox');
        }
    }


    /**
     * Method create model form
     *
     * @param $model Base_Doctrine_Record
     * @param $formClassName string
     */
    protected function _form($model, $formClassName, $options = array())
    {
        $formClassName = Setting::getFormClass($formClassName);
        $form = new $formClassName(array('model' => $model));
        $form_name = isset($options['form_name']) ? $options['form_name'] : 'form';
        $preSave = isset($options['preSave']) ? $options['preSave'] : null;
        $postSave = isset($options['postSave']) ? $options['postSave'] : null;
        $callback_function = isset( $options['callback_function'] ) ? $options['callback_function'] : '';

        if ( $this->_request->isPost()){
            if($form->isValid( $this->_request->getPost())){
                $isNew = $model->isNew();

                if($preSave && method_exists($this, $preSave)){
                    $this->$preSave($model, $form);
                }

                $model->save();

                if($postSave && method_exists($this, $postSave)){
                    $this->$postSave($model, $form);
                }

                if(isset($form->id)){ $form->id->setValue($model->getId()); }

                if($this->_request->isXmlHttpRequest()){
                    $returnData = array(
                        'html' => $form->render(),
                        'gritter_title' => $this->view->translate('gritter-title_record_save'),
                        'gritter_text' => $this->view->translate('gritter-text_record_save'),
                        'callback_function' => $callback_function,
                    );

                    if($isNew){
                        $returnData['location'] = $this->view->url($this->_getReturnParams(), $this->_route_return);
                    }

                    $this->_helper->json($returnData);
                }else{
                    $this->_flash()->success->addMessage('label_cms_save_success');
                    $this->_redirector()->gotoRouteAndExit($this->_getReturnParams(), $this->_route_return);
                }

            }elseif($this->_request->isXmlHttpRequest()){
                $this->_helper->json(array(
                    'html' => $form->render(),
                    'gritter_title' => $this->view->translate('gritter-title_validation_error'),
                    'gritter_text' => $this->view->translate('gritter-text_validation_error'),
                ));
            }
        }

        $this->view->$form_name = $form;
        $this->_helper->viewRenderer('admin/form', null, array('noController' => true));

        return $form;
    }


    private function _getReturnUrl()
    {
        $url = null;
        if($this->getParam('return')){ $url = $this->getParam('return');}
        elseif(isset($_SERVER['HTTP_REFERER'])){ $url = $_SERVER['HTTP_REFERER']; }

        if(empty($url)){ $url = Base::url('admin'); }

        return $url;
    }

    /**
     * @return array
     */
    protected function _getReturnParams()
    {
        return array();
    }



    /**
     * QUERY ACTIONS AND METHOD
     */

    public function saveQueryAction()
    {
        if(!$this->_queryFilterClassName){
            throw new Exception('Class name is required');
        }

        if(!class_exists($this->_queryFilterClassName)){
            throw new Exception('Class Not Exists: '.$this->_queryFilterClassName);
        }
        $filter = new $this->_queryFilterClassName();

        if(!method_exists($this->_queryFilterClassName, 'isFilter') || !$filter->isFilter()){
            throw new Exception('Form not a filter: '.$this->_queryFilterClassName);
        }

        $queryData['filter'] = $filter->getValues(true);
        $queryData['column'] = array();

        $queryModel = new Query();

        $queryModel->form = $filter->getElementsBelongTo();
        $queryModel->id_user = Base_Auth::getUser('id_user');
        $queryModel->data = json_encode($queryData);

        $this->_form($queryModel, 'News_Form_Admin_Query');
        $this->view->placeholder( 'page-title' )->set( $this->view->translate('cms-page-title_query_save') );
    }


    protected function _prepareQueryData()
    {
        $queryOptions = array('form' => $this->_queryFilterClassName);

        $this->_queryList = Query::getList($queryOptions);

        if ($this->hasParam('id_query') && isset($this->_queryList[$this->getParam('id_query')]))
        {
            $queryModel = $this->_queryList[$this->getParam('id_query')];
            $this->forward403Unless( $queryModel );

            $queryData = json_decode($queryModel['data'], true);

            $this->_listData['filter'] = $queryData['filter'];
            $this->_listData['id_query'] = $queryModel['id_query'];

        }

        $this->view->queryList = $this->_queryList;
        $this->view->listData = $this->_listData;
    }




    /**
     * MASS ACTIONS METHODS
     */

    protected function _massActionForm()
    {
        if(!$this->_modelMassActions){
            return;
        }

        $massActionForm = new Admin_Form_MassAction(array('actions' => $this->_modelMassActions));
        $url = $this->view->url(array(
            'module' => $this->_request->getModuleName(), 'controller' => $this->_request->getControllerName()),
            'admin_mass-action'
        );

        $this->view->massActionForm = $massActionForm;
        $this->view->massActionUrl = $url;

        $this->view->headScript()->appendFile('/assets/js/massEdit.js');
    }

    public function massAction()
    {
        if($this->_modelMassName === null){
            throw new Exception('Not specified model name for MassAction in ' . get_class($this) . '.');
        }

        $records = array();
        $params = $this->getAllParams();
        $modelMassName = $this->_modelMassName;
        $massActionForm = new Admin_Form_MassAction();

        $massActionParams = $params['massAction'];
        $action = $params[$massActionForm->getElementsBelongTo()]['mass'];

        foreach ($massActionParams as $mass)
        {
            if ($mass['id']){
                $records[$mass['id']] = $modelMassName::findRecordAll($mass['id']);
            }
        }

        $this->{$action . 'MassAction'}($records,$massActionParams);

        $this->_flash()->success->addMessage('message_mass_action_'.$action);
        $this->_redirector()->gotoUrlAndExit($this->_getReturnUrl());
    }

    public function saveMassAction ($records = null, $massActionParams = null){}

    public function deleteMassAction($records = null, $massActionParams = null)
    {
        foreach ($records as $record) {
            $record->delete();
        }
    }

    public function activeMassAction($records = null, $massActionParams = null)
    {
        foreach ($records as $record) {
            $record->setIsActive( true );
            $record->save();
        }
    }

    public function disActiveMassAction($records = null, $massActionParams = null)
    {
        foreach ($records as $record) {
            $record->setIsActive( false );
            $record->save();
        }
    }

    public function archiveMassAction($records = null, $massActionParams = null)
    {
        foreach ($records as $record) {
            $record->setArchivedAt( date("Y-m-d H:i:s") );
            $record->save();
        }
    }

    public function unarchiveMassAction($records = null, $massActionParams = null)
    {
        foreach ($records as $record) {
            $record->setArchivedAt( null );
            $record->save();
        }
    }
}

