<?php


abstract class Base_Doctrine_Record extends Doctrine_Record
{
    /**
     * @var integer
     */
    protected $_typeLog;

    /**
     * @var array
     */
    protected $_logData = array();


     /**
     * @return int|null
     */
    public function getId()
    {
        $identifier = $this->getTable()->getIdentifier();
        return $this->$identifier;
    }


    /**
     * @return bool
     */
    public function isNew()
    {
        $id = $this->getId();
        return null === $id ? true : false;
    }


    /**
     * @return boolean
     */
    public function isActive()
    {
        return $this->is_active;
    }


    /**
     * @param array $options
     * @return Doctrine_Query
     */
    public static function getQuery($options = array())
    {
        $tableName = isset($options['_tableName']) ? $options['_tableName'] : get_called_class();
        $table = Doctrine::getTable($tableName); /** @var $table Base_Doctrine_Table **/
        $hydrate = isset($options['hydrate']) ? $options['hydrate'] : Doctrine::HYDRATE_ARRAY;
        $identifier = (array) $table->getIdentifier();

        $optionsSelect = $table->getOption('select');
        $select = 'o.*';

        if(isset($options['select'])) {
            $select = $options['select'];
        }elseif($hydrate == Doctrine::HYDRATE_ARRAY){
            if(!empty($optionsSelect)){
                $select = $optionsSelect;
            }
        }


        $coll_key = $table->getOption('coll_key');
        $coll_key = isset($options['coll_key']) ? 'INDEXBY o.'.$options['coll_key'] : (!empty($coll_key) ? 'INDEXBY o.'.$coll_key : '');

        $query = Doctrine_Query::create()
            ->select($select)
            ->from($tableName . ' o ' . $coll_key);


        $query->setHydrationMode($hydrate);

        if ( $table->hasField('is_active', false) ) {
            if ( !array_key_exists('is_active', $options) ) {
                $query->addWhere('o.is_active = 1');
            } elseif ( strlen( $options['is_active'] ) > 0 ) {
                $query->addWhere('o.is_active = ?', $options['is_active']);
            }
        }

        if ( $table->hasField('archived_at', false) ) {
            if ( !array_key_exists('archived_at', $options) || !strlen( $options['archived_at'] )) {
                $query->addWhere('o.archived_at IS NULL');
            } elseif ( strlen( $options['archived_at'] ) > 1 ) {
                $query->addWhere('(o.archived_at IS NULL OR o.archived_at >= ?)', $options['archived_at']);
            }elseif ($options['archived_at'] === '1') {
                $query->addWhere('o.archived_at IS NOT NULL');
            }
        }

        if ( isset( $options['limit'] ) ) {
            $query->limit($options['limit']);
        }


        if(isset($options['id'])){
            if ( is_array( $options['id'] ) ) {
                $query->whereIn("o.$identifier[0]", $options['id']);
            } else {
                $query->addWhere("o.$identifier[0] = ?", $options['id']);
            }
        }


        $id_language = isset($options['id_language']) ? $options['id_language'] : Base_I18n::getLangId();

        if ( $table->hasRelation('Translations') ) {
            if($select == 'o.*'){ $query->addSelect('t.*'); }
            $trLeftJoin = isset($options['translations_leftJoin']) && $options['translations_leftJoin'] ? 'left' : 'inner';
            $query->{$trLeftJoin.'Join'}('o.Translations t WITH t.id_language = ? INDEXBY t.id_language', $id_language);
        } else if ( $table->hasField('id_language', false) && $tableName != 'Language' && $tableName != 'User' ) {
            $query->addWhere('o.id_language = ?', $id_language);
        }


        if(isset($options['search']) && ($search = $table->getOption('search')) != ''){
            $query->addWhere("$search LIKE ?", '%' . $options['search'] . '%');
        }

        if(isset($options['order']) && ($options['order'])  != ''){
            $query->orderBy($options['order']);
        }

        if(isset($options['debug'])){
            var_dump($query->getSqlQuery());
            exit();
        }

        return $query;
    }


    /**
     * @param array $options
     * @return bool|Doctrine_Record|mixed
     * @throws Doctrine_Query_Exception
     */
    public static function getRecord($options = array())
    {
        $options['limit'] = 1;
        if(!isset($options['_class'])){ $options['_class'] = get_called_class(); }

        $query = $options['_class']::getQuery($options);
        $result = $query->execute();

        if ( count( $result ) ) {
            if ( is_array( $result ) ) {
                return array_shift($result);
            } else {
                return $result->getFirst();
            }
        }

        return false;
    }


    /**
     * @param $id
     * @param array $options
     * @return bool|Doctrine_Record|mixed
     * @throws Exception
     */
    public static function find($id, $options = array())
    {
        if(empty($id)){
            throw new Exception('ID value is required in find method');
        }

        $options['id'] = $id;
        if(!isset($options['_class'])){ $options['_class'] = get_called_class(); }

        return $options['_class']::getRecord($options);
    }


    /**
     * @param $id
     * @param array $options
     * @return bool|Doctrine_Record
     * @throws Exception
     */
    public static function findRecord($id, $options = array())
    {
        if(empty($id)){
            throw new Exception('ID value is required in find method');
        }

        $options['hydrate'] = Doctrine::HYDRATE_RECORD;
        if(!isset($options['_class'])){ $options['_class'] = get_called_class(); }

        return $options['_class']::find($id, $options);
    }


    /**
     * @param $id
     * @param array $options
     * @return bool|Doctrine_Record
     */
    public static function findRecordAll($id, $options = array())
    {
        $options['is_active'] = '';
        $options['archived_at'] = '0';
        $options['translations_leftJoin'] = true;
        if(!isset($options['_class'])){ $options['_class'] = get_called_class(); }

        return  $options['_class']::findRecord($id, $options);
    }


    /**
     * @param array $options
     * @return Doctrine_Collection|array
     * @throws Doctrine_Query_Exception
     */
    public static function getList($options = array())
    {
        if(!isset($options['_class'])){ $options['_class'] = get_called_class(); }

        return $options['_class']::getQuery($options)->execute();
    }

    /**
     * @param string $method
     * @param array $args
     * @return mixed
     * @throws Doctrine_Record_UnknownPropertyException
     */
    public function __call($method, $args)
    {
        $lcMethod = substr($method, 0, 3);
        $field = Base::camelToLine(substr($method, 3));

        $langKey = $lcMethod == 'set' ? 1 : 0;
        $id_language = isset($args[$langKey]) ? $args[$langKey] : Base_I18n::getLangId();
        array_unshift($args, $field);
        array_push($args, $id_language);

        if($this->getTable()->hasField($field)){
            $lcMethod = '_'.$lcMethod.'Field';
            return call_user_func_array( array($this, $lcMethod), $args);
        }else{
            return parent::__call($method, $args);
        }

    }


    /**
     * @param $method
     * @param $args
     * @return mixed
     * @throws Doctrine_Record_UnknownPropertyException
     */
    public static function __callStatic($method, $args)
    {
        $lcMethod = strtolower($method);
        $table = Doctrine::getTable(get_called_class());

        if (substr($lcMethod, 0, 6) == 'findby' || substr($lcMethod, 0, 9) == 'findoneby') {
            return call_user_func_array( array($table, $method), $args);
        }else{
            throw new Doctrine_Record_UnknownPropertyException(sprintf('Unknown method %s::%s', get_class(get_called_class()), $method));
        }
    }


    /**
     * @param $field
     * @param $id_language
     * @param array $values
     * @return mixed
     * @throws Exception
     */
    protected function _getField($field, $id_language, $values = array())
    {
        if(isset($this->$field)){
            return $this->$field;
        }elseif(isset($this->getTransaltion($id_language)->$field)){
            return $this->getTransaltion($id_language)->$field;
        }else{
            throw new Exception(sprintf('Unknown field (column) %s::%s', get_class($this), $field));
        }
    }


    /**
     * @param $field
     * @param $value
     * @param $id_language
     * @param array $values
     * @return $this
     * @throws Exception
     */
    protected function _setField($field, $value, $id_language, $values = array())
    {
        if(isset($this->$field)){
            $this->$field = $value;
        }elseif(isset($this->getTransaltion($id_language)->$field)){
            $this->getTransaltion($id_language)->$field = $value;
        }else{
            throw new Exception(sprintf('Unknown field (column) %s::%s', get_class($this), $field));
        }

        return $this;
    }


    /**
     * @param null $id_language
     * @return mixed
     * @throws Exception
     */
    public function getTransaltion($id_language = null)
    {
        $id_language = null === $id_language ? Base_I18n::getLangId() : $id_language;

        if(!isset($this->Translations[$id_language]) && !$this->isNew()){
            $table = get_called_class().'Translation';

            $translation = self::find($this->getId(), array(
                'hydrate' => Doctrine::HYDRATE_RECORD, 'id_language' =>  $id_language, '_tableName' => $table));

            if($translation){
                $this->Translations[$id_language] = $translation;
            }
        }

        return $this->Translations[$id_language];
    }


    /**
     * @param $values
     */
    public function setFields($values)
    {
        $table = $this->getTable();
        foreach($values as $k => $v){
            if($table->hasField($k)){
                $method = 'set'.Base::lineToCamel($k);
                $this->$method($v, null, $values);
            }
        }
    }


    /**
     * @param $value
     * @param null $id_language
     * @return $this
     */
    public function setArchivedAt($value, $id_language = null)
    {
        if((bool) $value && $this->archived_at === null){
            $this->archived_at = date("Y-m-d H:i:s");
        }

        if((bool) !$value && $this->archived_at !== null){
            $this->archived_at = NULL;
        }

        return $this;
    }


    /**
     * @param $event
     */
    public function preDelete($event)
    {
        $this->_typeLog = Log::TYPE_DELETE;
        $this->preLogData();
        parent::preDelete($event);
    }


    /**
     * @param $event
     */
    public function preSave($event)
    {
        $this->preLogData();
        parent::preSave($event);
    }


    /**
     * @param $event
     */
    public function postSave($event)
    {
        parent::postSave($event);
        $this->postLogData();
    }


    /**
     * @param $event
     */
    public function postDelete($event)
    {
        parent::postDelete($event);
        $this->postLogData();
    }


    /**
     * Preparation of data to log
     */
    protected function preLogData()
    {
        if((bool)Setting::getSetting('system.log_on') && $this->option('log') && empty($this->_logData))
        {
            $this->_logData = $this->getLogData();
            $refKeys = array_keys($this->getReferences());

            foreach($refKeys as $related){
                if(isset($this[$related])){
                    if($related == 'Translations'){
                        $record = $this->getTransaltion();
                    }else{
                        $record = $this->$related;
                    }

                    if($record instanceof Base_Doctrine_Record){
                        $relatedResult = $record->getLogData();
                        if(!empty($relatedResult)){
                            $this->_logData[$related] = $relatedResult;
                        }
                    }
                }
            }
        }
    }


    /**
     * Add logs to the database
     * @throws Exception
     */
    protected function postLogData()
    {
        if(!empty($this->_logData)){
            // save log
            $options = array();
            $options['model'] = get_class($this);
            $options['id_object'] = $this->getId();

            $options['relations'] = $this->getLogRelations();

            Log::addLog($this->_typeLog, $this->_logData, $options);

            $this->_logData = array();
        }
    }


    /**
     * @return array
     * @throws Doctrine_Record_State_Exception
     */
    protected function getLogData()
    {
        $result = array();
        $this->option('log', false);

        if(empty($this->_typeLog)){
            if($this->state() == Doctrine_Record::STATE_TDIRTY) {
                $this->_typeLog = Log::TYPE_CREATE;
            }elseif($this->state() == Doctrine_Record::STATE_TCLEAN) {
                $this->_typeLog = Log::TYPE_DELETE;
            }else {
                $this->_typeLog = Log::TYPE_MODIFIED;
            }
        }

        if ($this->_typeLog == Log::TYPE_MODIFIED) {
            $preMod = (array) $this->getModified(true, false);
            $postMod = (array) $this->getModified(false, false);

            foreach($preMod as $k => $v){
                $result[$k] = array($v, $postMod[$k]);
            }
        }elseif($this->_typeLog == Log::TYPE_CREATE) {
            $result += array_filter($this->toArray(0));
        }elseif($this->_typeLog == Log::TYPE_DELETE){
            $result += $this->toArray(0);
        }

        $identifier = (array)$this->getTable()->getIdentifier();
        foreach($identifier as $v){
            unset($result[$v]);
        }

        if(strstr('Translations', get_class($this))){
            $result['id_language'] = $this['id_language'];
        }

        if(!empty($result)){
            $result = array('_type' => $this->_typeLog) + $result;
        }

        return $result;
    }


    /**
     * @return array
     */
    protected function getLogRelations()
    {
        return array();
    }
}