<?php

class Base_Layout
{

    private static $registred_layouts = array();
    private static $current_layout = array();

    private static $_template = null;
    private static $_template_type = 'frontend';

    private static $_basicScripts = null;

    /**
     * Method provide functionality of defining layout files, devided by modules
     * @static
     * @param $module
     * @param $filename
     * @param array $placeholders
     * @param array $cms_placeholders_definitions
     * @throws Exception
     */
    public static function registerLayout($module, $filename, $is_edit = true, $placeholders = array() )
    {
        if (isset(self::$registred_layouts[$module][$filename])) {
            return;
            throw new Exception("Alias already taken, please choose another one.");
        }
        self::$registred_layouts[$module][$filename]['is_edit'] = $is_edit;
        self::$registred_layouts[$module][$filename]['placeholders'] = $placeholders;
    }


    public static function loadLayouts()
    {
        $cache = Base_Cache::getCache('default');
        $cache_name = 'layouts';
        self::$registred_layouts = $cache->load( $cache_name );

        if(self::$registred_layouts === false ){
            $layouts = Layout::getList();

            foreach($layouts as $layout){
                self::registerLayout($layout['module'], $layout['filename'], $layout['is_edit'], json_decode($layout['placeholders'], true));
            }

            $cache->save(self::$registred_layouts, $cache_name);
        }
    }


    public static function renderPlaceholders()
    {
        $definitions = self::getLayoutDefinitions();
        foreach($definitions as $k => $v ){
            if(isset($v['widgets'])){
                foreach($v['widgets'] as $kk => $vv){
                    if(class_exists($vv['_class'])){
                        $widget = new $vv['_class'];
                        $widget_response = $widget->render($vv);
                        self::placeholder($k, $kk, $widget_response);
                    }
                }
            }
        }
    }


    public static function layoutIsRegistred($module, $filename)
    {
        return isset(self::$registred_layouts[$module][$filename]);
    }

    /**
     * Method by which you can select the layout. Layout must be pre-registered.
     * @see Base_Layout::registerLayout
     *
     * @static
     * @param $module
     * @param $filename
     * @throws Exception
     */
    public static function setLayout($module, $filename = 'layout')
    {
        $view = self::getView();

        if (!isset(self::$registred_layouts[$module][$filename])) {
            throw new Exception("Layout '$filename' for module '$module' is not registred.\n");
        }
        self::$current_layout = array('module' => $module, 'filename' => $filename);
        $module = Base::camelToDash($module);

        if(null === self::$_basicScripts){
            self::$_basicScripts = $view->getScriptPaths();
        }else{
            $view->setScriptPath(array_reverse(self::$_basicScripts));
        }


        /** @var $layout Zend_Layout */
        $layout = $view->layout();

        if(!preg_grep('/modules(.*)scripts/', $view->getScriptPaths())){
            $view->addBasePath(APPLICATION_PATH . '/modules/' . $module . '/views/');
        }

        $view->addBasePath(APPLICATION_PATH . '/modules/' . $module . '/layouts/');
        $layout->setLayout($filename);

        if(null === self::$_template){
            self::$_template = Setting::getSetting('template.' . self::$_template_type);
        }

        if(!empty(self::$_template)){
            $template_path = APPLICATION_PATH . '/templates/' . self::$_template_type . '/'. self::$_template;
            $view->addBasePath($template_path.DS.'views');

            $front_module_name = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();
            $view->addBasePath($template_path . '/modules/' . $front_module_name . '/views');
            self::getView()->addBasePath($template_path . '/modules/' . $module . '/layouts');
        }
    }

    public static function getTemplate()
    {
        return self::$_template;
    }

    public static function setTemplate($template = null, $type = null)
    {
        if(!empty($type)){
            self::setTemplateType($type);
        }

        self::$_template = $template;
    }

    public static function getTemplateType()
    {
        return self::$_template_type;
    }

    public static function setTemplateType($type = null)
    {
        self::$_template_type = $type;
    }

    /**
     * Method witch you can check whether placeholder exists in current layout.
     *
     * @param $placeholder
     * @return bool
     */
    public static function placeholderExists($placeholder)
    {
        $module = @self::$current_layout['module'];
        $filename = @self::$current_layout['filename'];
        $placeholders = @self::$registred_layouts[$module][$filename]['placeholders'];
        return is_array($placeholders) && array_key_exists($placeholder, $placeholders);

    }

    public static function disableLayout()
    {
        self::getView()->layout()->disableLayout();
    }

    public static function disableView()
    {
        Zend_Controller_Front::getInstance()->setParam('noViewRenderer', true);
    }


    /**
     * Alias for plceholder View_Helper
     *
     * @static
     * @param $placeholder
     * @param int $order
     * @param $value
     * @throws Exception
     */
    public static function placeholder($placeholder, $order = 0, $value)
    {
        if (!self::placeholderExists($placeholder)) {
            throw new Exception("There is no registred placeholder '$placeholder' in layout " .
                self::$current_layout['filename'] . " (module: " . self::$current_layout['module'] . ")");
        }
        self::getView()->placeholder($placeholder)->{$placeholder . $order} = $value;
    }

    /**
     * Get current Zend_View object
     *
     * @static
     * @return Zend_View
     */
    public static function getView()
    {
        /** @var $bootstrap Bootstrap */
        $bootstrap = Zend_Controller_Front::getInstance()->getParam('bootstrap');

        /** @var $layout Zend_Layout */
        $layout = $bootstrap->getResource('layout');
        return $layout->getView();

    }

    public static function getLayoutDefinitions($module = null, $filename = null)
    {
        if(!$module){ $module = self::$current_layout['module']; }
        if(!$filename){ $filename = self::$current_layout['filename']; }

        if (!isset(self::$registred_layouts[$module][$filename])) {
            throw new Exception("Layout '$filename' for module '$module' is not registred.\n");
        }
        if ($filename !== null) {
            return self::$registred_layouts[$module][$filename]['placeholders'];
    }
    }

    public static function setLayoutDefinitions($placeholders, $module = null, $filename = null)
    {
        if(!$module){ $module = self::$current_layout['module']; }
        if(!$filename){ $filename = self::$current_layout['filename']; }

        if (!isset(self::$registred_layouts[$module][$filename])) {
            throw new Exception("Layout '$filename' for module '$module' is not registred.\n");
        }

        self::$registred_layouts[$module][$filename]['placeholders'] = $placeholders;
    }

    public static function prependWidget($widget, $placeholder, $module = null, $filename = null)
    {
        if(!$module){ $module = self::$current_layout['module']; }
        if(!$filename){ $filename = self::$current_layout['filename']; }

        if (!isset(self::$registred_layouts[$module][$filename])) {
            throw new Exception("Layout '$filename' for module '$module' is not registred.\n");
        }

        if (!self::placeholderExists($placeholder)) {
            throw new Exception("There is no registred placeholder '$placeholder' in layout " .
            self::$current_layout['filename'] . " (module: " . self::$current_layout['module'] . ")");
        }

        array_unshift(self::$registred_layouts[$module][$filename]['placeholders'][$placeholder]['widgets'], $widget);
    }



    public static function getLayoutCmsNames($module, $editable = 1)
    {
        if (!isset(self::$registred_layouts[$module])) {
            throw new Exception("Module '$module' has not registred any layouts yet.\n");
        }
        $return = array();
        foreach (self::$registred_layouts[$module] as $layout_name => $layout) {
            if($editable == 'all' || $layout['is_edit'] == (bool) $editable){
                $return[$module . '-' . $layout_name] = $layout_name;
            }
        }
        return $return;

    }

    public static function getAllLayoutCmsNames()
    {
        $return = array();
        foreach (self::$registred_layouts as $module_name => $layout){
            $ret = self::getLayoutCmsNames($module_name);
            if(!empty($ret)){
                $return[$module_name] = $ret;
            }
        }
        return $return;
    }


    public static function getWidgetScriptPaths($module)
    {
        $result = array();
        $view = self::getView();
        $scripts = $view->getScriptPaths();

        $patterns = array();
        $replacements = array();

        $patterns[0] = '/(.*modules\/)(.*)((layouts|views).*)(scripts)(.*)/';
        $replacements[0] = '$1'.$module.'/$3widgets$6';

        $patterns[1] = '/(.*)(scripts)(.*)/';
        $replacements[1] = '$1widgets$3';

        foreach($scripts as $v){
            $result[] = preg_replace($patterns, $replacements, $v);
        }

        return $result;
    }


    public static function getRegistredLayouts()
    {
        return self::$registred_layouts;
    }


}