<?php


class Base_Mail extends Zend_Mail
{
    /**
     * @var array
     */
    private $_options;

    /**
     * @var Zend_Mail_Transport_Smtp
     */
    private $_transporter = null;


    public function  __construct($options = array(), $transporter = null)
    {
        parent::__construct( 'utf-8' );

        $options = $options + Setting::getEmailSetting();
        $this->setOptions( $options );

        if(null !== $transporter){
            $this->setTransporter($transporter);
        }else{
            if( $this->_options['smtp_on'] ){
                $config = array(
                    'auth' => 'login',
                    'username' => $this->_options['username'],
                    'password' => $this->_options['password'],
                );


                $extra_keys = array('ssl','port');
                foreach($extra_keys as $key)
                {
                    if(isset($options[$key]) && !empty($options[$key])){
                        $config[$key] = $options[$key];
                    }
                }
                $this->setTransporter(new Zend_Mail_Transport_Smtp( $this->_options['transporter'], $config));
            }
            $this->setFrom( $this->_options['email_from'], $this->_options['name_from'] );
        }
    }

    /**
     * Method return Mail Transporter
     *
     * @return Zend_Mail_Transport_Smtp
     */
    public function getTransporter()
    {
        return $this->_transporter;
    }

    /**
     * Method set Mail Transporter
     *
     * @param Zend_Mail_Transport_Smtp $transporter
     * @return $this
     */
    public function setTransporter(Zend_Mail_Transport_Smtp $transporter)
    {
        $this->_transporter = $transporter;

        return $this;
    }


    public function setOptions( $options )
    {
        $this->_options = $options;
    }

    public function setBodyContent($template)
    {
        $view = Base_Layout::getView();
        $view->mail_content = $view->render($template);
        $this->setBodyHtml( $view->render('_mail_layout.phtml'),'utf-8');
        $this->setBodyText( strip_tags( str_replace(array('<br/>','<br>'), "\r\n", $view->mail_content ) ) ,'utf-8');

//        echo $view->render('_mail_layout.phtml');
//        exit();
    }


    public function send( $send = null, $option = array())
    {
        $ret = false;
        if( null == $send ) {
            $send = !Setting::getSetting(Setting::EMAIL_CRON_ON);
        }
        $emailSend = isset($option['model']) ? $option['model'] : null;
        $db_save = isset($option['db_save']) ? $option['db_save'] : true;

        if(!$emailSend)
        {
            $emailSend = new EmailSend();
            $emailSend->to_send = true;
            $emailSend->subject = $this->getSubject();
            $emailSend->from = $this->getFrom();
            $recipients = $this->getRecipients();
            $emailSend->to = $recipients[0];
            $emailSend->mail = serialize($this);
        }

        if( $send ) {
            if( $this->_options['send_on'] ){
                try{
                    $ret = parent::send( $this->_transporter );
                    $emailSend->is_sent = true;
                    $emailSend->to_send = false;
                    $emailSend->sent_time = date('YmdHis');
                }catch (Exception $e){
                    if(DEBUG){
                        throw new Exception($e);
                    }else{
                        $emailSend->exception = $e;
                    }
                }
            }else{
                $emailSend->to_send = false;
            }
        }

        if($db_save) {
            $emailSend->save();
        }

        return $ret;
    }



}
